package com.hz.pm.api.common.statemachine.action;

import cn.hutool.core.lang.Assert;
import com.hz.pm.api.common.statemachine.builder.impl.AdaptStateMachineBuilderImpl;
import com.hz.pm.api.common.statemachine.event.AdaptStateChangeEvent;
import com.hz.pm.api.common.statemachine.util.TenderStateMachineUtil;
import com.hz.pm.api.projectdeclared.model.entity.Purchase;
import com.hz.pm.api.projectlib.model.enumeration.status.TenderAdaptStatus;
import lombok.extern.slf4j.Slf4j;
import org.springframework.messaging.Message;
import org.springframework.statemachine.annotation.OnTransition;
import org.springframework.statemachine.annotation.WithStateMachine;

/**
 * <p>
 * 测试验证状态机action集合类
 * </p>
 *
 * @author WendyYang
 * @since 17:08 2024/2/24
 */
@Slf4j
@WithStateMachine(id = AdaptStateMachineBuilderImpl.MACHINE_ID)
public class AdaptStateChangeAction {

    private Purchase getPurchaseInfo(Message<AdaptStateChangeEvent> message) {
        Purchase purchase = (Purchase) message.getHeaders().get(TenderStateMachineUtil.PURCHASE);
        Assert.notNull(purchase, "未获取到需要状态变更的标段信息");
        return purchase;
    }

    @OnTransition(source = "WITHOUT_ADAPT_INFO", target = "ADAPT_INFO_AUDIT")
    public void SUBMIT_ADAPT_INFO(Message<AdaptStateChangeEvent> message) {
        Purchase purchase = getPurchaseInfo(message);
        purchase.setAdaptStatus(TenderAdaptStatus.ADAPT_INFO_AUDIT.getCode());
    }

    @OnTransition(source = "ADAPT_INFO_AUDIT", target = "ADAPT_INFO_PASSED")
    public void ADAPT_INFO_PASSED(Message<AdaptStateChangeEvent> message) {
        Purchase purchase = getPurchaseInfo(message);
        purchase.setAdaptStatus(TenderAdaptStatus.ADAPT_INFO_PASSED.getCode());
    }

    @OnTransition(source = "ADAPT_INFO_AUDIT", target = "ADAPT_INFO_FAILED")
    public void ADAPT_INFO_FAILED(Message<AdaptStateChangeEvent> message) {
        Purchase purchase = getPurchaseInfo(message);
        purchase.setAdaptStatus(TenderAdaptStatus.ADAPT_INFO_FAILED.getCode());
    }

    @OnTransition(source = "ADAPT_INFO_FAILED", target = "ADAPT_INFO_AUDIT")
    public void RESUBMIT_ADAPT_INFO(Message<AdaptStateChangeEvent> message) {
        Purchase purchase = getPurchaseInfo(message);
        purchase.setAdaptStatus(TenderAdaptStatus.ADAPT_INFO_AUDIT.getCode());
    }

}
