package com.hz.pm.api.common.statemachine.action;

import cn.hutool.core.lang.Assert;
import com.hz.pm.api.common.statemachine.builder.impl.TenderStateMachineBuilderImpl;
import com.hz.pm.api.common.statemachine.event.ProjectStateChangeEvent;
import com.hz.pm.api.common.statemachine.util.TenderStateMachineUtil;
import com.hz.pm.api.projectdeclared.model.entity.Purchase;
import com.hz.pm.api.projectlib.model.enumeration.status.TenderStatus;
import lombok.extern.slf4j.Slf4j;
import org.springframework.messaging.Message;
import org.springframework.statemachine.annotation.OnTransition;
import org.springframework.statemachine.annotation.WithStateMachine;

/**
 * <p>
 * 标段状态机action集合类
 * </p>
 *
 * @author WendyYang
 * @since 17:08 2024/2/24
 */
@Slf4j
@WithStateMachine(id = TenderStateMachineBuilderImpl.MACHINE_ID)
public class TenderStateChangeAction {

    private Purchase getPurchaseInfo(Message<ProjectStateChangeEvent> message) {
        Purchase purchase = (Purchase) message.getHeaders().get(TenderStateMachineUtil.PURCHASE);
        Assert.notNull(purchase, "未获取到需要状态变更的标段信息");
        return purchase;
    }

    /**
     * 填写采购信息
     *
     * @param message \
     */
    @OnTransition(source = "TO_BE_SUBMIT_PURCHASE_INFO", target = "TO_BE_SUBMIT_CONSTRUCTION_INFO")
    public void SUBMIT_PURCHASE_INFO(Message<ProjectStateChangeEvent> message) {
        Purchase purchase = getPurchaseInfo(message);
        purchase.setStatus(TenderStatus.TO_BE_SUBMIT_CONSTRUCTION_INFO.getCode());
    }

    /**
     * 填写合同信息
     *
     * @param message \
     */
    @OnTransition(source = "TO_BE_SUBMIT_CONSTRUCTION_INFO", target = "TO_BE_SUBMIT_OPERATION_PLAN")
    public void SUBMIT_CONSTRUCTION_INFO(Message<ProjectStateChangeEvent> message) {
        Purchase purchase = getPurchaseInfo(message);
        purchase.setStatus(TenderStatus.TO_BE_SUBMIT_OPERATION_PLAN.getCode());
    }

    /**
     * 填写实施计划
     *
     * @param message \
     */
    @OnTransition(source = "TO_BE_SUBMIT_OPERATION_PLAN", target = "TO_BE_SUBMIT_FIRST_INSPECTED_INFO")
    public void SUBMIT_OPERATION_PLAN(Message<ProjectStateChangeEvent> message) {
        Purchase purchase = getPurchaseInfo(message);
        purchase.setStatus(TenderStatus.TO_BE_SUBMIT_FIRST_INSPECTED_INFO.getCode());
    }

    /**
     * 上传初验材料
     *
     * @param message \
     */
    @OnTransition(source = "TO_BE_SUBMIT_FIRST_INSPECTED_INFO", target = "TO_BE_SUBMIT_FINALLY_INSPECTED_APPLY")
    public void SUBMIT_FIRST_INSPECTED_FILES(Message<ProjectStateChangeEvent> message) {
        Purchase purchase = getPurchaseInfo(message);
        purchase.setStatus(TenderStatus.TO_BE_SUBMIT_FINALLY_INSPECTED_APPLY.getCode());
    }

    /**
     * 终验申报
     *
     * @param message \
     */
    @OnTransition(source = "TO_BE_SUBMIT_FINALLY_INSPECTED_APPLY", target = "ON_FINALLY_INSPECTED_APPLY")
    public void SUBMIT_FINALLY_INSPECTED(Message<ProjectStateChangeEvent> message) {
        Purchase purchase = getPurchaseInfo(message);
        purchase.setStatus(TenderStatus.ON_FINALLY_INSPECTED_APPLY.getCode());
    }

    /**
     * 终验申报重新提交
     *
     * @param message \
     */
    @OnTransition(source = "FINALLY_INSPECTED_FAILED", target = "ON_FINALLY_INSPECTED_APPLY")
    public void RESUBMIT_FINALLY_INSPECTED(Message<ProjectStateChangeEvent> message) {
        Purchase purchase = getPurchaseInfo(message);
        purchase.setStatus(TenderStatus.ON_FINALLY_INSPECTED_APPLY.getCode());
    }

    /**
     * 终验申报通过
     *
     * @param message \
     */
    @OnTransition(source = "ON_FINALLY_INSPECTED_APPLY", target = "FINALLY_INSPECTED_PASSED")
    public void FINALLY_INSPECTED_PASSED(Message<ProjectStateChangeEvent> message) {
        Purchase purchase = getPurchaseInfo(message);
        purchase.setStatus(TenderStatus.FINALLY_INSPECTED_PASSED.getCode());
    }

    /**
     * 终验申报拒绝
     *
     * @param message \
     */
    @OnTransition(source = "ON_FINALLY_INSPECTED_APPLY", target = "FINALLY_INSPECTED_FAILED")
    public void FINALLY_INSPECTED_FAILED(Message<ProjectStateChangeEvent> message) {
        Purchase purchase = getPurchaseInfo(message);
        purchase.setStatus(TenderStatus.FINALLY_INSPECTED_FAILED.getCode());
    }

}
