package com.hz.pm.api.common.statemachine.event;

import com.hz.pm.api.common.statemachine.action.ProjectStateChangeAction;
import com.hz.pm.api.projectlib.model.enumeration.status.ProjectStatus;
import lombok.AllArgsConstructor;
import lombok.Getter;
import org.springframework.messaging.Message;

/**
 * <p>
 * 项目状态变更事件
 * </p>
 *
 * @author WendyYang
 * @since 20:20 2024/4/2
 */
@Getter
@AllArgsConstructor
public enum ProjectStateChangeEvent implements AbstractStateChangeEvent {

    /**
     * @see ProjectStateChangeAction#PROJECT_APPLICATION_SUBMIT(Message)
     */
    PROJECT_APPLICATION_SUBMIT(ProjectStatus.TO_BE_DECLARED.getCode(), null, null),

    /**
     * @see ProjectStateChangeAction#COMPLIANCE_REVIEW_FAILED(Message)
     */
    COMPLIANCE_REVIEW_FAILED(null, ProjectStatus.ON_COMPLIANCE_REVIEW.getCode(), null),

    /**
     * @see ProjectStateChangeAction#COMPLIANCE_REVIEW_RESUBMIT(Message)
     */
    COMPLIANCE_REVIEW_RESUBMIT(ProjectStatus.COMPLIANCE_REVIEW_FAILED.getCode(), null, null),

    /**
     * @see ProjectStateChangeAction#COMPLIANCE_REVIEW_PASS(Message)
     */
    COMPLIANCE_REVIEW_PASS(ProjectStatus.ON_COMPLIANCE_REVIEW.getCode(), null, null),

    /**
     * @see ProjectStateChangeAction#PROJECT_REVIEW_SUBMIT(Message)
     */
    PROJECT_REVIEW_SUBMIT(ProjectStatus.WITHOUT_PROJECT_REVIEW.getCode(), null, null),

    /**
     * @see ProjectStateChangeAction#PROJECT_REVIEW_RESUBMIT(Message)
     */
    PROJECT_REVIEW_RESUBMIT(ProjectStatus.PROJECT_REVIEW_FAILED.getCode(), null, null),

    /**
     * @see ProjectStateChangeAction#PROJECT_REVIEW_FAILED(Message)
     */
    PROJECT_REVIEW_FAILED(null, ProjectStatus.ON_PROJECT_REVIEW.getCode(), null),

    /**
     * @see ProjectStateChangeAction#PROJECT_REVIEW_PASS(Message)
     */
    PROJECT_REVIEW_PASS(ProjectStatus.ON_PROJECT_REVIEW.getCode(), null, null),

    /**
     * @see ProjectStateChangeAction#ANNUAL_PLAN_PASS(Message)
     */
    ANNUAL_PLAN_PASS(ProjectStatus.ON_ANNUAL_PLAN.getCode(), null, null),

    /**
     * 立项备案提交
     *
     * @see ProjectStateChangeAction#DECLARED_RECORD_SUBMIT(Message)
     */
    DECLARED_RECORD_SUBMIT(ProjectStatus.DECLARED_APPROVED_TO_BE_RECORD.getCode(), null, null),

    /**
     * 立项备案重新提交
     *
     * @see ProjectStateChangeAction#DECLARED_RECORD_RESUBMIT(Message)
     */
    DECLARED_RECORD_RESUBMIT(ProjectStatus.DECLARED_APPROVED_RECORD_FAILED.getCode(), null, null),

    /**
     * 立项备案通过
     * @see ProjectStateChangeAction#DECLARED_RECORD_PASS(Message)
     */
    DECLARED_RECORD_PASS(ProjectStatus.DECLARED_APPROVED_RECORD_AUDITING.getCode(), null, null),

    /**
     * 立项备案不通过
     * @see ProjectStateChangeAction#DECLARED_RECORD_FAILED(Message)
     */
    DECLARED_RECORD_FAILED(null, ProjectStatus.DECLARED_APPROVED_RECORD_AUDITING.getCode(), null),

    //==================================================================================================================

    /**
     * 发布采购公告
     * @see ProjectStateChangeAction#SUBMIT_PURCHASE_NOTICE(Message)
     */
    SUBMIT_PURCHASE_NOTICE(ProjectStatus.TO_BE_PURCHASED.getCode(), null, null),

    /**
     * 填写实施计划
     * @see ProjectStateChangeAction#SUBMIT_OPERATION_PLAN(Message)
     */
    SUBMIT_OPERATION_PLAN(ProjectStatus.ON_PURCHASING.getCode(), null, null),

    /**
     * 上传初验材料
     * @see ProjectStateChangeAction#SUBMIT_FIRST_INSPECTED_FILES(Message)
     */
    SUBMIT_FIRST_INSPECTED_FILES(ProjectStatus.TO_BE_FIRST_INSPECTED.getCode(), null, null),
    /**
     * 终验申请(项目状态变为：终验审核中)
     * @see ProjectStateChangeAction#FINAL_ACCEPTANCE_APPLICATION(Message)
     */
    FINAL_ACCEPTANCE_APPLICATION(ProjectStatus.ON_PILOT_RUNNING.getCode(), null, null),
    /**
     * 终验审核不通过(项目状态变为：终验审核不通过)
     * @see ProjectStateChangeAction#FINAL_ACCEPTANCE_REJECT(Message)
     */
    FINAL_ACCEPTANCE_REJECT(null, ProjectStatus.ON_FINALLY_INSPECTED.getCode(), null),
    /**
     * 终验审核通过(项目状态变为：已归档)
     * @see ProjectStateChangeAction#FINAL_ACCEPTANCE_PASS(Message)
     */
    FINAL_ACCEPTANCE_PASS(ProjectStatus.ON_FINALLY_INSPECTED.getCode(), null, null),

    FINAL_ACCEPTANCE_IS_UNDER_REVIEW_WITHDRAW(null, null, ProjectStatus.ON_FINALLY_INSPECTED.getCode()),

    /**
     * 重新提交 终验
     * @see ProjectStateChangeAction#FINAL_RE_ACCEPTANCE_APPLICATION(Message)
     */
    FINAL_RE_ACCEPTANCE_APPLICATION(ProjectStatus.FINALLY_INSPECTED_FAILED.getCode(), null, null);


    private final Integer passStatusCode;
    private final Integer rejectStatusCode;
    private final Integer withdrawStatusCode;

}
