package com.hz.pm.api.common.util;

import org.springframework.scheduling.concurrent.ThreadPoolTaskExecutor;
import org.springframework.scheduling.concurrent.ThreadPoolTaskScheduler;

import java.util.concurrent.ThreadPoolExecutor;

/**
 * <p>
 * ThreadPoolUtil
 * </p>
 *
 * @author WendyYang
 * @since 2023/10/27
 **/
public class ThreadPoolUtil {

    private ThreadPoolUtil() {
    }

    public static final ThreadPoolTaskExecutor REQUEST;
    public static final ThreadPoolTaskScheduler SCHEDULER;

    static {
        REQUEST = new MDCThreadPoolTaskExecutor();
        REQUEST.setCorePoolSize(5);
        REQUEST.setMaxPoolSize(15);
        REQUEST.setQueueCapacity(300);
        REQUEST.setThreadPriority(Thread.NORM_PRIORITY);
        REQUEST.setKeepAliveSeconds(120);
        REQUEST.setThreadNamePrefix("requestAsyncExecutor-");
        REQUEST.setRejectedExecutionHandler(new ThreadPoolExecutor.CallerRunsPolicy());
        REQUEST.initialize();
    }

    static {
        SCHEDULER = new MDCThreadPoolTaskScheduler();
        SCHEDULER.setPoolSize(5);
        SCHEDULER.setThreadPriority(Thread.NORM_PRIORITY - 1);
        SCHEDULER.setWaitForTasksToCompleteOnShutdown(true);
        SCHEDULER.setThreadNamePrefix("scheduler-");
        SCHEDULER.setRejectedExecutionHandler(new ThreadPoolExecutor.CallerRunsPolicy());
        SCHEDULER.initialize();
    }

}
