package com.hz.pm.api.projectdeclared.controller;

import com.hz.pm.api.common.util.ExcelDownUtil;
import com.hz.pm.api.projectdeclared.manage.DeclaredProjectManage;
import com.hz.pm.api.projectdeclared.model.dto.DeclaredProjectListParamDTO;
import com.hz.pm.api.projectdeclared.model.dto.DefaultDeclaredDTO;
import com.hz.pm.api.projectdeclared.model.dto.ProjectDraftSaveDTO;
import com.hz.pm.api.projectdeclared.model.enumerization.ProjectDraftTypeEnum;
import com.hz.pm.api.projectdeclared.model.vo.ProjectDraftVO;
import com.hz.pm.api.projectlib.model.req.ProjectListReq;
import com.hz.pm.api.projectlib.model.vo.ProjectLibListItemVO;
import com.ningdatech.basic.model.PageVo;
import com.ningdatech.log.annotation.WebLog;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;

/**
 * <p>
 * DeclaredProjectController
 * </p>
 *
 * @author WendyYang
 * @since 16:22 2024/3/29
 */
@Slf4j
@Validated
@RestController
@RequestMapping("/api/v1/declared")
@Api(value = "DeclaredProject", tags = "申报管理-申报项目")
@RequiredArgsConstructor
public class DeclaredProjectController {

    private final DeclaredProjectManage declaredProjectManage;

    @ApiOperation(value = "申报项目已申报列表", notes = "申报项目已申报列表")
    @GetMapping("/list")
    public PageVo<ProjectLibListItemVO> list(@ModelAttribute ProjectListReq req) {
        return declaredProjectManage.projectLibList(req);
    }

    @ApiOperation(value = "申报项目草稿箱列表", notes = "申报项目草稿箱列表")
    @GetMapping("/draft")
    public PageVo<ProjectDraftVO> draft(@ModelAttribute DeclaredProjectListParamDTO params) {
        return declaredProjectManage.pageDraft(params, ProjectDraftTypeEnum.PROJECT_DECLARED.getCode());
    }

    @ApiOperation(value = "立项备案草稿箱列表", notes = "立项备案草稿箱列表")
    @GetMapping("/declaredRecord/draft")
    public PageVo<ProjectDraftVO> declaredRecordDraft(@ModelAttribute DeclaredProjectListParamDTO params) {
        return declaredProjectManage.pageDraft(params, ProjectDraftTypeEnum.DECLARED_RECORD.getCode());
    }

    @ApiOperation(value = "申报项目草稿箱详情", notes = "申报项目草稿箱详情")
    @GetMapping("/draft/{id}")
    public ProjectDraftVO draft(@PathVariable(value = "id") Long id) {
        return declaredProjectManage.draftDetail(id);
    }

    @ApiOperation(value = "申报项目保存至草稿箱", notes = "申报项目保存至草稿箱")
    @WebLog("申报项目保存至草稿箱")
    @PostMapping("/save-to-draft")
    public Object saveToDraft(@Validated @RequestBody ProjectDraftSaveDTO dto) {
        return declaredProjectManage.saveToDraft(dto, ProjectDraftTypeEnum.PROJECT_DECLARED);
    }

    @ApiOperation(value = "立项备案保存至草稿箱", notes = "立项备案保存至草稿箱")
    @WebLog("立项备案保存至草稿箱")
    @PostMapping("/declaredRecord/save-to-draft")
    public Object declaredRecordSaveToDraft(@Validated @RequestBody ProjectDraftSaveDTO dto) {
        return declaredProjectManage.saveToDraft(dto, ProjectDraftTypeEnum.DECLARED_RECORD);
    }

    @ApiOperation(value = "申报项目", notes = "申报项目")
    @WebLog("申报项目")
    @PostMapping("/start")
    public String startTheProcess(@Validated @RequestBody DefaultDeclaredDTO dto) {
        String instanceId = declaredProjectManage.startTheProcess(dto);
        return "启动流程实例 【" + instanceId + "】 成功";
    }

    @ApiOperation(value = "重新申报项目", notes = "重新申报项目")
    @WebLog("重新申报项目")
    @PostMapping("/restart")
    public String reStartTheProcess(@Validated @RequestBody DefaultDeclaredDTO dto) {
        return declaredProjectManage.reStartTheProcess(dto);
    }

    @GetMapping("/export")
    @ApiOperation("申报项目列表导出")
    @WebLog("申报项目列表导出")
    public void exportList(ProjectListReq req, HttpServletResponse response) {
        ExcelDownUtil.downXls(response, req, declaredProjectManage::exportList);
    }

    @ApiOperation(value = "初始化状态变更记录", hidden = true)
    @GetMapping("/initStateChangeRecord")
    public void initStateChangeRecord() {
        declaredProjectManage.initStateChangeRecord();
    }

}
