package com.hz.pm.api.projectdeclared.controller;

import com.hz.pm.api.common.util.ExcelDownUtil;
import com.hz.pm.api.projectdeclared.manage.PurchaseManage;
import com.hz.pm.api.projectdeclared.model.dto.PurchaseSaveDTO;
import com.hz.pm.api.projectdeclared.model.req.*;
import com.hz.pm.api.projectdeclared.model.req.SubmitPurchaseNoticeReq.PurchaseNoticeTenderDTO;
import com.hz.pm.api.projectdeclared.model.vo.PurchaseVO;
import com.hz.pm.api.projectdeclared.model.vo.TestValidListVO;
import com.hz.pm.api.projectdeclared.model.vo.XcfhxApplyListVO;
import com.hz.pm.api.projectlib.model.req.ProjectListReq;
import com.hz.pm.api.projectlib.model.vo.ProjectLibListItemVO;
import com.ningdatech.basic.model.PageVo;
import com.ningdatech.log.annotation.WebLog;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.util.List;

/**
 * @Classname PurchaseController
 * @Description 申报项目-采购结果备案
 * @Date 2023/5/29 09:29
 * @Author PoffyZhang
 */
@Slf4j
@Validated
@RestController
@RequestMapping("/api/v1/declared/purchase")
@Api(value = "PurchaseController", tags = "申报管理-采购结果备案")
@RequiredArgsConstructor
public class PurchaseController {
    private final PurchaseManage purchaseManage;

    @ApiOperation(value = "待采购结果备案列表", notes = "待采购结果备案列表")
    @GetMapping("/project-list")
    public PageVo<ProjectLibListItemVO> projectList(@ModelAttribute ProjectListReq req) {
        return purchaseManage.projectLibList(req);
    }

    @ApiOperation(value = "采购结果备案详情", notes = "采购结果备案详情")
    @GetMapping("/detail/{projectId}")
    public List<PurchaseVO> detailByProjectId(@PathVariable Long projectId) {
        return purchaseManage.detailByProjectId(projectId);
    }

    @GetMapping("/export")
    @ApiOperation("待采购结果备案列表导出")
    @WebLog("待采购结果备案列表导出")
    public void exportList(ProjectListReq req, HttpServletResponse response) {
        ExcelDownUtil.downXls(response, req, purchaseManage::exportList);
    }

    @ApiOperation(value = "填写采购结果")
    @WebLog("填写采购结果")
    @PostMapping("/submit-result")
    public String submitResult(@Validated @RequestBody PurchaseSaveDTO dto) {
        return purchaseManage.submitResult(dto);
    }

    @ApiOperation("发布采购公告")
    @WebLog("发布采购公告")
    @PostMapping("/notice/submit")
    public void submitPurchaseNotice(@RequestBody SubmitPurchaseNoticeReq req) {
        purchaseManage.submitPurchaseNotice(req);
    }

    @ApiOperation("采购信息备案")
    @WebLog("采购信息备案")
    @PostMapping("/record/submit")
    public void submitPurchaseRecord(@RequestBody SubmitPurchaseRecordReq req) {
        purchaseManage.submitPurchaseRecord(req);
    }

    @GetMapping("/pageTestValid")
    @ApiOperation("获取测试验证列表")
    public PageVo<TestValidListVO> pageTestValid(TestValidListReq req) {
        return purchaseManage.testValidList(req);
    }

    @GetMapping("/pageSelfTest")
    @ApiOperation("获取系统自测列表")
    public PageVo<TestValidListVO> pageSelfTest(TestValidListReq req) {
        return purchaseManage.selfTestList(req);
    }

    @GetMapping("/pageAdaption")
    @ApiOperation("获取适配改造列表")
    public PageVo<TestValidListVO> pageAdaption(TestValidListReq req) {
        return purchaseManage.adaptList(req);
    }

    @ApiOperation("提交测试验证信息")
    @WebLog("提交测试验证信息")
    @PostMapping("/submitTestValidInfo")
    public void submitTestValidInfo(@RequestBody TestValidInfoReq req) {
        purchaseManage.submitTestValidInfo(req);
    }

    @ApiOperation("提交适配改造信息")
    @WebLog("提交适配改造信息")
    @PostMapping("/submitAdaptionInfo")
    public void submitAdaptionInfo(@RequestBody TestValidInfoReq req) {
        purchaseManage.submitAdaptionInfo(req);
    }

    @ApiOperation("提交系统自测信息")
    @WebLog("提交系统自测信息")
    @PostMapping("/submitSelfTestInfo")
    public void submitSelfTestInfo(@RequestBody TestValidInfoReq req) {
        purchaseManage.submitSelfTestInfo(req);
    }

    @GetMapping("/pageXcfhxApply")
    @ApiOperation("获取信创符合性申请列表")
    public PageVo<XcfhxApplyListVO> pageXcfhxApply(XcfhxListReq req) {
        return purchaseManage.pageXcfhxApply(req);
    }

    @GetMapping("/purchaseIntention/{unitId}")
    @ApiOperation("获取采购意向")
    public List<PurchaseNoticeTenderDTO> listPurchaseIntention(@PathVariable Long unitId) {
        return purchaseManage.listPurchaseIntention(unitId);
    }

}
