package com.hz.pm.api.projectdeclared.manage;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.io.FileUtil;
import cn.hutool.core.io.IoUtil;
import cn.hutool.json.JSONUtil;
import com.alibaba.excel.EasyExcel;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.google.common.collect.Lists;
import com.hz.pm.api.common.enumeration.ProjectProcessType;
import com.hz.pm.api.common.helper.UserInfoHelper;
import com.hz.pm.api.common.model.constant.BizConst;
import com.hz.pm.api.common.statemachine.util.ProjectStateMachineUtil;
import com.hz.pm.api.common.statemachine.util.TenderStateMachineUtil;
import com.hz.pm.api.common.statemachine.util.XcfhxStateMachineUtil;
import com.hz.pm.api.common.util.DecimalUtil;
import com.hz.pm.api.common.util.ExcelDownUtil;
import com.hz.pm.api.common.util.ExcelExportStyle;
import com.hz.pm.api.common.util.StrUtils;
import com.hz.pm.api.external.MhFileClient;
import com.hz.pm.api.projectdeclared.model.dto.DeclaredProjectExportDTO;
import com.hz.pm.api.projectdeclared.model.dto.PaymentPlanSupplementDTO;
import com.hz.pm.api.projectdeclared.model.dto.PreInsSaveDTO;
import com.hz.pm.api.projectdeclared.model.entity.*;
import com.hz.pm.api.projectdeclared.model.req.SubmitConstructionReq;
import com.hz.pm.api.projectdeclared.model.req.XcfhxApplyReq;
import com.hz.pm.api.projectdeclared.model.vo.ContractVO;
import com.hz.pm.api.projectdeclared.model.vo.PreInsAcceptancePersonVO;
import com.hz.pm.api.projectdeclared.model.vo.PreInsVO;
import com.hz.pm.api.projectdeclared.model.vo.ProjectContractListVO;
import com.hz.pm.api.projectdeclared.service.*;
import com.hz.pm.api.projectlib.helper.ProjectManageUtil;
import com.hz.pm.api.projectlib.model.entity.Project;
import com.hz.pm.api.projectlib.model.entity.ProjectAnnualPaymentPlan;
import com.hz.pm.api.projectlib.model.entity.ProjectInst;
import com.hz.pm.api.projectlib.model.enumeration.ProjectTypeNewEnum;
import com.hz.pm.api.projectlib.model.enumeration.status.ProjectStatus;
import com.hz.pm.api.projectlib.model.enumeration.status.TenderStatus;
import com.hz.pm.api.projectlib.model.enumeration.status.TenderXcfhxApplyStatus;
import com.hz.pm.api.projectlib.model.req.ProjectListReq;
import com.hz.pm.api.projectlib.model.vo.ProjectLibListItemVO;
import com.hz.pm.api.projectlib.model.vo.TenderListInfoVO;
import com.hz.pm.api.projectlib.service.IProjectAnnualPaymentPlanService;
import com.hz.pm.api.projectlib.service.IProjectInstService;
import com.hz.pm.api.projectlib.service.IProjectService;
import com.hz.pm.api.sys.manage.ProcessModelManage;
import com.hz.pm.api.user.security.model.UserFullInfoDTO;
import com.hz.pm.api.user.security.model.UserInfoDetails;
import com.hz.pm.api.user.util.LoginUserUtil;
import com.ningdatech.basic.exception.BizException;
import com.ningdatech.basic.function.VUtils;
import com.ningdatech.basic.model.ApiResponse;
import com.ningdatech.basic.model.PageVo;
import com.ningdatech.basic.util.CollUtils;
import com.ningdatech.basic.util.NdDateUtils;
import com.ningdatech.file.controller.FileController;
import com.ningdatech.file.entity.vo.result.FileResultVO;
import com.ningdatech.file.service.FileService;
import com.wflow.bean.entity.WflowModels;
import com.wflow.workflow.bean.dto.OrgInfoDTO;
import com.wflow.workflow.bean.vo.ProcessStartParamsVo;
import com.wflow.workflow.service.ProcessInstanceService;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.io.IOUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.mock.web.MockMultipartFile;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import javax.servlet.http.HttpServletResponse;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.math.BigDecimal;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.*;
import java.util.concurrent.atomic.AtomicInteger;
import java.util.stream.Collectors;

/**
 * @Classname ContructionManage
 * @Description
 * @Date 2023/5/29 14:48
 * @Author PoffyZhang
 */
@Component
@Slf4j
@RequiredArgsConstructor
public class ConstructionManage {

    private final IProjectService projectService;
    private final UserInfoHelper userInfoHelper;
    private final IPurchaseService purchaseService;
    private final IPurchaseInstService purchaseInstService;
    private final IContractService contractService;
    private final ProcessModelManage processModelManage;
    private final IProjectInstService projectInstService;
    private final IPaymentPlanService paymentPlanService;
    private final DeclaredProjectHelper declaredProjectHelper;
    private final IPreInsAcceptancePersonService acceptancePersonService;
    private final ProcessInstanceService processInstanceService;
    private final ProjectStateMachineUtil projectStateMachineUtil;
    private final IProjectAnnualPaymentPlanService projectPaymentPlanService;
    private final TenderStateMachineUtil tenderStateMachineUtil;
    private final XcfhxStateMachineUtil xcfhxStateMachineUtil;
    private final FileController fileController;
    private final MhFileClient mhFileClient;

    /**
     * 待采购的-项目列表
     *
     * @param req \
     * @return \
     */
    public PageVo<ProjectLibListItemVO> projectLibList(ProjectListReq req) {
        UserInfoDetails user = LoginUserUtil.loginUserDetail();
        LambdaQueryWrapper<Project> query = ProjectManageUtil.projectQuery(req);
        //建设中状态
        query.isNull(Project::getContractAmount);
        query.eq(Project::getStage, ProjectStatus.PROJECT_APPROVED.getCode());
        query.eq(Project::getNewest, Boolean.TRUE);
        //只能看自己单位的
        query.eq(Project::getBuildOrgCode, user.getMhUnitIdStr());
        query.orderByAsc(Project::getTransactionTime);
        Page<Project> page = projectService.page(req.page(), query);
        long total;
        if ((total = page.getTotal()) == 0) {
            return PageVo.empty();
        }
        List<ProjectLibListItemVO> retData = CollUtils.convert(page.getRecords(), w -> {
            ProjectLibListItemVO item = new ProjectLibListItemVO();
            item.setId(w.getId());
            item.setProjectName(w.getProjectName());
            item.setCreateOn(w.getCreateOn());
            item.setDeclaredAmount(w.getDeclareAmount());
            item.setStage(w.getStage());
            item.setStatus(w.getStatus());
            item.setProjectType(w.getProjectType());
            item.setProjectYear(w.getProjectYear());
            item.setBuildOrg(w.getBuildOrgName());
            item.setBizDomain(w.getBizDomain());
            item.setProcessStatus(w.getProcessStatus());
            item.setInstCode(w.getInstCode());
            item.setIsHigherSuperOrg(w.getIsHigherSuperOrg());
            item.setApprovedAmount(w.getApprovalAmount());
            item.setApprovalDate(w.getApprovalDate());
            item.setTransactionAmount(w.getTransactionAmount());
            item.setTransactionTime(w.getTransactionTime());
            return item;
        });
        return PageVo.of(retData, total);
    }

    /**
     * 已完善合同信息的列表
     *
     * @param req \
     * @return \
     */
    public PageVo<ProjectContractListVO> projectListPerfect(ProjectListReq req) {
        UserFullInfoDTO user = userInfoHelper.getUserFullInfo(LoginUserUtil.getUserId());
        LambdaQueryWrapper<Project> query = ProjectManageUtil.projectQuery(req);
        query.eq(Project::getNewest, Boolean.TRUE);
        //只能看自己单位的
        query.eq(Project::getBuildOrgCode, user.getMhUnitIdStr());
        query.orderByAsc(Project::getUpdateOn);
        Page<Project> page = projectService.page(req.page(), query);

        if (CollUtil.isEmpty(page.getRecords())) {
            return PageVo.empty();
        }

        List<Project> projects = page.getRecords();
        List<String> projectCodes = projects.stream().map(Project::getProjectCode)
                .collect(Collectors.toList());
        List<PaymentPlan> paymentPlans = paymentPlanService.list(Wrappers.lambdaQuery(PaymentPlan.class)
                .in(PaymentPlan::getProjectCode, projectCodes));
        Map<String, List<PaymentPlan>> paymentMap = paymentPlans.stream()
                .collect(Collectors.groupingBy(PaymentPlan::getProjectCode));

        List<ProjectContractListVO> records = CollUtils.convert(page.getRecords(), w -> {
            ProjectContractListVO item = BeanUtil.copyProperties(w, ProjectContractListVO.class);
            item.setBuildOrg(w.getBuildOrgName());
            item.setSupplemented(checkIsSupplement(w, paymentMap));
            return item;
        });
        return PageVo.of(records, page.getTotal());
    }

    /**
     * 判断 是否需要补充
     *
     * @param w          \
     * @param paymentMap \
     * @return \
     */
    private Boolean checkIsSupplement(Project w, Map<String, List<PaymentPlan>> paymentMap) {
        if (paymentMap.containsKey(w.getProjectCode())) {
            List<PaymentPlan> paymentPlans = paymentMap.get(w.getProjectCode());
            if (CollUtil.isEmpty(paymentPlans)) {
                return Boolean.FALSE;
            }
            for (PaymentPlan plan : paymentPlans) {
                if (Objects.isNull(plan.getActualPaymentAmount())) {
                    return Boolean.FALSE;
                }
            }
            return Boolean.TRUE;
        }
        return Boolean.FALSE;
    }

    public void exportList(HttpServletResponse response, ProjectListReq param) {
        LambdaQueryWrapper<Project> query = ProjectManageUtil.projectQuery(param);
        //待采购状态
        query.isNull(Project::getContractAmount);
        query.eq(Project::getStage, ProjectStatus.PROJECT_APPROVED.getCode());
        query.eq(Project::getNewest, Boolean.TRUE);
        query.orderByAsc(Project::getTransactionTime);
        List<Project> records = projectService.list(query);

        AtomicInteger serialNumber = new AtomicInteger(0);
        List<DeclaredProjectExportDTO> collect = Lists.newArrayList();
        if (CollUtil.isNotEmpty(records)) {
            List<Long> projectIds = records.stream().map(Project::getId).collect(Collectors.toList());

            List<Purchase> purchases = purchaseService.listByProjectIds(projectIds);
            Map<Long, Purchase> purchaseMap = purchases.stream().collect(Collectors.toMap(Purchase::getProjectId, v -> v));

            records.forEach(r -> {
                DeclaredProjectExportDTO exportData = new DeclaredProjectExportDTO();
                BeanUtils.copyProperties(r, exportData);
                exportData.setProjectTypeName(ProjectTypeNewEnum.getDesc(r.getProjectType()));
                exportData.setStatusName(ProjectStatus.getDesc(r.getStatus()));
                String createOnStr = NdDateUtils.format(r.getCreateOn(), "yyyy-MM-dd HH:mm");
                exportData.setCreateOn(createOnStr);
                exportData.setSerialNumber(serialNumber.incrementAndGet());
                exportData.setApprovedAmount(r.getApprovalAmount());
                exportData.setApprovalDate(r.getApprovalDate().format(DateTimeFormatter.ofPattern("yyyy-MM-dd")));
                if (purchaseMap.containsKey(r.getId())) {
                    Purchase purchase = purchaseMap.get(r.getId());
                    exportData.setTransactionAmount(purchase.getTransactionAmount());
                    exportData.setTransactionTime(purchase.getTransactionTime().format(DateTimeFormatter.ofPattern("yyyy-MM-dd")));
                }
                collect.add(exportData);
            });
        }

        String fileName = "合同备案项目列表";
        ExcelDownUtil.setFileName(fileName, response);
        //数据导出处理函数
        try {
            EasyExcel.write(response.getOutputStream(), DeclaredProjectExportDTO.class)
                    .autoCloseStream(false)
                    .registerWriteHandler(ExcelExportStyle.formalStyle())
                    .sheet(fileName)
                    .doWrite(collect);
        } catch (Exception e) {
            throw new BizException(e);
        }
    }

    /**
     * 获取合同备案详情
     *
     * @param projectId \
     * @return \
     */
    public List<ContractVO> detailContractByProjectId(Long projectId) {
        List<Contract> contracts = contractService.list(Wrappers.lambdaQuery(Contract.class)
                .eq(Contract::getProjectId, projectId));
        return BeanUtil.copyToList(contracts, ContractVO.class);
    }

    /**
     * 获取合同备案详情
     *
     * @param bidIds \
     * @return \
     */
    public List<ContractVO> detailContractByBidIds(Collection<Long> bidIds) {
        if (CollUtil.isEmpty(bidIds)) {
            return Collections.emptyList();
        }
        List<Contract> contracts = contractService.list(Wrappers.lambdaQuery(Contract.class)
                .in(Contract::getBidId, bidIds));
        return BeanUtil.copyToList(contracts, ContractVO.class);
    }

    @Transactional(rollbackFor = Exception.class)
    public synchronized String submitPurchaseContract(SubmitConstructionReq req) {
        UserInfoDetails user = LoginUserUtil.loginUserDetail();
        Long projectId = req.getProjectId();
        Project project = projectService.getNewestNoNull(req.getProjectId());

        //首先要判断 项目当前状态 是不是 采购结果备案
        VUtils.isTrue(!ProjectStatus.ON_PURCHASING.getCode().equals(project.getStatus()) ||
                        !ProjectStatus.PROJECT_APPROVED.getCode().equals(project.getStage()))
                .throwMessage("提交失败 该项目不是 建设中或者已立项阶段");

        Purchase purchase = purchaseService.getById(req.getBidId());

        if (!TenderStatus.TO_BE_SUBMIT_CONSTRUCTION_INFO.eq(purchase.getStatus())) {
            throw BizException.wrap("该标段不支持填写合同信息");
        }
        tenderStateMachineUtil.pass(purchase);
        purchaseService.updateById(purchase);
        LocalDateTime now = LocalDateTime.now();
        Contract contract = new Contract();
        contract.setContractTime(req.getContractTime());
        contract.setAttachment(req.getAttachment());
        contract.setName(req.getName());
        contract.setProjectId(projectId);
        contract.setProjectCode(project.getProjectCode());
        contract.setBidId(req.getBidId());
        contract.setId(req.getConstructionId());
        contract.setCreateBy(user.getUserIdStr());
        contract.setUpdateBy(user.getUserIdStr());
        contract.setCreateOn(now);
        contract.setUpdateOn(now);
        contract.setTotalAmount(req.getTotalAmount());
        contract.setConstructionNo(req.getConstructionNo());
        contract.setSupplierContact(req.getSupplierContact());
        contract.setSupplierContactInfo(req.getSupplierContactInfo());
        contractService.saveOrUpdate(contract);

        LambdaQueryWrapper<Contract> query = Wrappers.lambdaQuery(Contract.class)
                .eq(Contract::getProjectId, projectId);
        List<Contract> allContracts = contractService.list(query);
        BigDecimal totalAmount = DecimalUtil.sum(allContracts, Contract::getTotalAmount);
        project.setUpdateOn(now);
        project.setContractAmount(totalAmount);
        projectService.updateById(project);
        return "填写成功";
    }

    private static final List<TenderStatus> PRE_INSPECTED_PURCHASE_STATUS;

    static {
        PRE_INSPECTED_PURCHASE_STATUS = new ArrayList<>();
        PRE_INSPECTED_PURCHASE_STATUS.add(TenderStatus.TO_BE_SUBMIT_FIRST_INSPECTED_INFO);
    }

    /**
     * 待初验项目列表
     *
     * @param req
     * @return
     */
    public PageVo<ProjectLibListItemVO> preProjectList(ProjectListReq req) {
        UserInfoDetails user = LoginUserUtil.loginUserDetail();
        Set<Long> projectIds = purchaseService.listProjectIdByStatus(PRE_INSPECTED_PURCHASE_STATUS);
        if (projectIds.isEmpty()) {
            return PageVo.empty();
        }
        LambdaQueryWrapper<Project> query = ProjectManageUtil.projectQuery(req);
        query.in(Project::getId, projectIds);
        query.eq(Project::getStage, ProjectStatus.PROJECT_APPROVED.getCode());
        query.eq(Project::getNewest, Boolean.TRUE);
        //只能看自己单位的
        query.eq(Project::getBuildOrgCode, user.getMhUnitIdStr());
        query.orderByAsc(Project::getDeliveryTime);
        Page<Project> page = projectService.page(req.page(), query);
        long total;
        if ((total = page.getTotal()) == 0) {
            return PageVo.empty();
        }
        List<Long> currPageProjectIds = CollUtils.fieldList(page.getRecords(), Project::getId);
        List<Purchase> purchases = purchaseService.listByProjectIds(currPageProjectIds);
        Map<Long, List<Purchase>> purchaseMap = CollUtils.group(purchases, Purchase::getProjectId);
        Map<Long, Contract> contractMap = contractService.listByProjectIds(projectIds);
        Map<Long, List<ProjectAnnualPaymentPlan>> paymentPlansMap = projectPaymentPlanService.listByProjectIds(projectIds);

        List<ProjectLibListItemVO> records = CollUtils.convert(page.getRecords(), w -> {
            ProjectLibListItemVO item = new ProjectLibListItemVO();
            item.setId(w.getId());
            item.setProjectName(w.getProjectName());
            item.setCreateOn(w.getCreateOn());
            item.setDeclaredAmount(w.getDeclareAmount());
            item.setStage(w.getStage());
            item.setStatus(w.getStatus());
            item.setProjectType(w.getProjectType());
            item.setProjectYear(w.getProjectYear());
            item.setBuildOrg(w.getBuildOrgName());
            item.setBizDomain(w.getBizDomain());
            item.setProcessStatus(w.getProcessStatus());
            item.setInstCode(w.getInstCode());
            item.setIsHigherSuperOrg(w.getIsHigherSuperOrg());
            item.setApprovedAmount(w.getApprovalAmount());
            item.setApprovalDate(w.getApprovalDate());
            item.setDeliveryTime(w.getDeliveryTime());
            item.setContractAmount(w.getContractAmount());
            List<ProjectAnnualPaymentPlan> paymentPlans = paymentPlansMap.getOrDefault(w.getId(), Collections.emptyList());
            BigDecimal totalAnnualAmount = paymentPlans.stream()
                    .map(ProjectAnnualPaymentPlan::getAnnualPlanAmount)
                    .reduce(BigDecimal::add).orElse(BigDecimal.ZERO);
            item.setAnnualPlanAmount(totalAnnualAmount);
            item.setTransactionTime(w.getTransactionTime());
            item.setTransactionAmount(w.getTransactionAmount());
            List<Purchase> currPurchases = purchaseMap.get(w.getId());
            if (currPurchases != null) {
                List<TenderListInfoVO> tmpPurchases = CollUtils.convert(currPurchases, x -> {
                    TenderListInfoVO tender = new TenderListInfoVO();
                    tender.setBidName(x.getBidName());
                    tender.setBidId(x.getId());
                    tender.setBidStatus(x.getStatus());
                    tender.setBidStatusName(TenderStatus.getDescByStatus(x.getStatus()));
                    Contract contract = contractMap.get(x.getId());
                    if (contract != null) {
                        tender.setConstructionAmount(contract.getTotalAmount());
                    }
                    return tender;
                });
                item.setTenders(tmpPurchases);
            }
            return item;
        });
        return PageVo.of(records, total);
    }

    public void exportPreList(HttpServletResponse response, ProjectListReq param) {
        UserFullInfoDTO user = userInfoHelper.getUserFullInfo(LoginUserUtil.getUserId());
        VUtils.isTrue(Objects.isNull(user)).throwMessage("获取登录用户失败!");
        LambdaQueryWrapper<Project> query = ProjectManageUtil.projectQuery(param);
        //待采购状态
        //只能看自己单位的
        query.eq(Project::getBuildOrgCode, user.getMhUnitId());
        query.eq(Project::getStage, ProjectStatus.PROJECT_APPROVED.getCode());
        query.eq(Project::getNewest, Boolean.TRUE);
        //交货时间 排序
        query.isNotNull(Project::getDeliveryTime);
        query.orderByAsc(Project::getDeliveryTime);
        List<Project> records = projectService.list(query);

        List<DeclaredProjectExportDTO> collect = Lists.newArrayList();
        if (CollUtil.isNotEmpty(records)) {
            List<Long> projectIds = new ArrayList<>();
            for (Project project : records) {
                projectIds.add(project.getId());
            }

            Map<Long, List<ProjectAnnualPaymentPlan>> paymentPlansMap = projectPaymentPlanService.listByProjectIds(projectIds);

            AtomicInteger serialNumber = new AtomicInteger(0);
            collect = records.stream().map(r -> {
                DeclaredProjectExportDTO exportDTO = new DeclaredProjectExportDTO();
                BeanUtils.copyProperties(r, exportDTO);
                exportDTO.setProjectTypeName(ProjectTypeNewEnum.getDesc(r.getProjectType()));
                exportDTO.setStatusName(ProjectStatus.getDesc(r.getStatus()));
                String createOnStr = NdDateUtils.format(r.getCreateOn(), "yyyy-MM-dd HH:mm");
                exportDTO.setCreateOn(createOnStr);
                exportDTO.setSerialNumber(serialNumber.incrementAndGet());
                exportDTO.setDeliveryTime(r.getDeliveryTime().format(DateTimeFormatter.ofPattern("yyyy-MM-dd")));
                exportDTO.setContractAmount(r.getContractAmount());

                List<ProjectAnnualPaymentPlan> paymentPlans = paymentPlansMap.getOrDefault(r.getId(), Collections.emptyList());
                BigDecimal totalAnnualAmount = paymentPlans.stream()
                        .map(ProjectAnnualPaymentPlan::getAnnualPlanAmount)
                        .reduce(BigDecimal::add).orElse(BigDecimal.ZERO);

                exportDTO.setAnnualPlanAmount(totalAnnualAmount);
                if (Objects.nonNull(r.getTransactionTime())) {
                    exportDTO.setTransactionTime(r.getTransactionTime().format(DateTimeFormatter.ofPattern("yyyy-MM-dd")));
                }
                exportDTO.setTransactionAmount(r.getTransactionAmount());
                return exportDTO;
            }).collect(Collectors.toList());
        }

        final String fileName = "合同备案项目列表";
        ExcelDownUtil.setFileName(fileName, response);
        //数据导出处理函数
        try {
            EasyExcel.write(response.getOutputStream(), DeclaredProjectExportDTO.class)
                    .autoCloseStream(false)
                    .registerWriteHandler(ExcelExportStyle.formalStyle())
                    .sheet(fileName)
                    .doWrite(collect);
        } catch (Exception e) {
            throw new BizException(e);
        }
    }

    /**
     * 查询初验信息
     *
     * @param projectId
     * @return
     */
    public List<PreInsVO> detailPreInsByProjectId(Long projectId) {
        List<Purchase> purchases = purchaseService.listByProjectId(projectId);
        List<Long> bidIds = CollUtils.fieldList(purchases, Purchase::getId);
        Map<Long, List<PreInsAcceptancePerson>> personMap = acceptancePersonService.listByBidIds(bidIds);
        return purchases.stream()
                .filter(w -> StrUtils.isNotBlank(w.getActualPerformanceIndicators()))
                .map(w -> {
                    PreInsVO vo = new PreInsVO();
                    vo.setBidId(w.getId());
                    vo.setProjectId(w.getProjectId());
                    vo.setPreliminaryInspectionMaterials(w.getPreliminaryInspectionMaterials());
                    vo.setIsCompletedLogCollection(w.getIsCompletedLogCollection());
                    vo.setActualPerformanceIndicators(w.getActualPerformanceIndicators());
                    List<PreInsAcceptancePerson> persons = personMap.get(w.getId());
                    if (persons != null) {
                        vo.setAcceptancePersons(convertPersons(persons));
                    }
                    return vo;
                }).collect(Collectors.toList());
    }

    private List<PreInsAcceptancePersonVO> convertPersons(List<PreInsAcceptancePerson> acceptancePersons) {
        if (CollUtil.isEmpty(acceptancePersons)) {
            return Collections.emptyList();
        }
        return acceptancePersons.stream()
                .map(u -> BeanUtil.copyProperties(u, PreInsAcceptancePersonVO.class))
                .collect(Collectors.toList());
    }

    @Transactional(rollbackFor = Exception.class)
    public synchronized void submitFirstInspectedFiles(PreInsSaveDTO req) {
        UserInfoDetails user = LoginUserUtil.loginUserDetail();
        Purchase purchase = purchaseService.getById(req.getBidId());
        if (!TenderStatus.TO_BE_SUBMIT_FIRST_INSPECTED_INFO.eq(purchase.getStatus())) {
            throw BizException.wrap("该标段无法上传初验材料");
        }
        LocalDateTime now = LocalDateTime.now();
        //初验人员
        if (Objects.nonNull(req.getAcceptancePersons())) {
            //先删除
            acceptancePersonService.removeByBidId(req.getBidId());
            if (CollUtil.isNotEmpty(req.getAcceptancePersons())) {
                List<PreInsAcceptancePerson> persons = req.getAcceptancePersons().stream().map(d -> {
                    PreInsAcceptancePerson person = new PreInsAcceptancePerson();
                    BeanUtil.copyProperties(d, person);
                    person.setBidId(req.getBidId());
                    person.setProjectId(req.getProjectId());
                    person.setCreateOn(now);
                    person.setUpdateOn(now);
                    person.setCreateBy(user.getUserIdStr());
                    person.setUpdateBy(user.getMhUnitIdStr());
                    return person;
                }).collect(Collectors.toList());
                acceptancePersonService.saveBatch(persons);
            }
        }

        tenderStateMachineUtil.pass(purchase);
        purchase.setPreliminaryInspectionMaterials(req.getPreliminaryInspectionMaterials());
        purchase.setIsCompletedLogCollection(req.getIsCompletedLogCollection());
        purchase.setActualPerformanceIndicators(req.getActualPerformanceIndicators());
        purchaseService.updateById(purchase);
        // 判断是否所有标段都上传了初验材料
        LambdaQueryWrapper<Purchase> purchaseQuery = Wrappers.lambdaQuery(Purchase.class)
                .eq(Purchase::getProjectId, req.getProjectId())
                .apply("LENGTH(NVL(preliminary_inspection_materials,'')) = 0")
                .last(BizConst.LIMIT_1);
        if (purchaseService.count(purchaseQuery) == 0) {
            Project project = projectService.getById(req.getProjectId());
            projectStateMachineUtil.pass(project);
            projectService.updateById(project);
        }
    }

    @Transactional(rollbackFor = Exception.class)
    public synchronized void submitXcfhxApply(XcfhxApplyReq req) {
        Purchase purchase = purchaseService.getById(req.getBidId());
        if (!TenderXcfhxApplyStatus.TO_BE_SUBMIT_XCFHX_APPLY.eq(purchase.getXcfhxApplyStatus())
                && !TenderXcfhxApplyStatus.XCFHX_APPLY_FAILED.eq(purchase.getXcfhxApplyStatus())) {
            throw BizException.wrap("该标段不支持提交信创符合性申请");
        }

        UserInfoDetails user = LoginUserUtil.loginUserDetail();
        WflowModels model = processModelManage.getWflowModelsNoNull(ProjectProcessType.XC_APPROVAL_PROCESS,
                user.getMhUnitId());

        ProcessStartParamsVo processParam = new ProcessStartParamsVo();
        processParam.setUser(declaredProjectHelper.buildUser(user));
        processParam.setProcessUsers(Collections.emptyMap());
        processParam.setFormData(Collections.emptyMap());
        // 获取发起单位、发起单位主管单位、发起单位上级主管条线单位信息
        Project project = projectService.getById(req.getProjectId());
        Map<String, OrgInfoDTO> orgMap = declaredProjectHelper.buildOrgModelMap(user, project);
        String instanceId = processInstanceService.startProcessLs(model, processParam, orgMap);

        ProjectInst projectInst = new ProjectInst();
        projectInst.setInstCode(instanceId);
        projectInst.setInstType(ProjectProcessType.XC_APPROVAL_PROCESS);
        projectInst.setProjectId(req.getProjectId());
        projectInstService.save(projectInst);
        log.info("信创审查提交成功 【{}】", instanceId);

        purchase.setXcfhxApplyFiles(req.getXcfhxApplyFiles());
        purchase.setMatchXcfhx(req.getMatchXcfhx());
        purchase.setXcfhxApplyRemark(req.getXcfhxApplyRemark());
        if (Boolean.TRUE.equals(req.getMatchXcfhx())) {
            if (StrUtils.isBlank(purchase.getMhXcfhxReportFile()) ||
                    req.getMhXcfhxReportFile().equals(purchase.getMhXcfhxReportFile())) {
                ApiResponse<FileResultVO> retFileInfo;
                File tmpFile = null;
                FileInputStream fis = null;
                try {
                    tmpFile = mhFileClient.downloadToTmpFile(req.getMhXcfhxReportFile());
                    String fileName = purchase.getBidName() + "-信创符合性测评报告." + FileUtil.getSuffix(tmpFile);
                    fis = new FileInputStream(tmpFile);
                    String mimeType = FileUtil.getMimeType(tmpFile.getPath());
                    MockMultipartFile multipartFile = new MockMultipartFile(fileName, fileName, mimeType, fis);
                    retFileInfo = fileController.upload(multipartFile, "default");
                } catch (IOException e) {
                    log.error("信创报告上传失败", e);
                    throw BizException.wrap("信创符合性测评报告上传失败");
                } finally {
                    if (tmpFile != null) {
                        tmpFile.deleteOnExit();
                    }
                    if (fis != null) {
                        IOUtils.closeQuietly(fis);
                    }
                }
                purchase.setXcfhxReportFiles(JSONUtil.toJsonStr(retFileInfo));
            }
            purchase.setMhXcfhxReportRecordId(req.getMhXcfhxReportRecordId());
            purchase.setMhXcfhxReportFile(req.getMhXcfhxReportFile());
        }

        xcfhxStateMachineUtil.pass(purchase);
        purchaseService.updateById(purchase);

        PurchaseInst purchaseInst = new PurchaseInst();
        purchaseInst.setBidId(purchase.getId());
        purchaseInst.setInstCode(instanceId);
        purchaseInst.setInstType(ProjectProcessType.XC_APPROVAL_PROCESS.getCode());
        purchaseInst.setProjectId(req.getProjectId());
        purchaseInstService.save(purchaseInst);
    }

    /**
     * 补充项目 合同 实际支付信息
     *
     * @param plans \
     * @return \
     */
    public String supplement(List<PaymentPlanSupplementDTO> plans) {
        if (CollUtil.isEmpty(plans)) {
            throw new BizException("入参");
        }

        for (PaymentPlanSupplementDTO plan : plans) {
            PaymentPlan paymentPlan = paymentPlanService.getById(plan.getId());
            VUtils.isTrue(Objects.isNull(paymentPlan))
                    .throwMessage("保存失败 该支付信息不存在 :" + plan.getId());
            VUtils.isTrue(Objects.nonNull(paymentPlan.getActualPaymentAmount()))
                    .throwMessage("保存失败 该支付信息已经保存过 实际支付金额 :" + plan.getId() + "," + paymentPlan.getActualPaymentAmount());
            paymentPlan.setActualPaymentAmount(plan.getActualPaymentAmount());
            paymentPlanService.updateById(paymentPlan);
        }

        return "保存成功";
    }
}
