package com.hz.pm.api.projectdeclared.model.entity;

import com.baomidou.mybatisplus.annotation.IdType;
import com.baomidou.mybatisplus.annotation.TableId;
import com.baomidou.mybatisplus.annotation.TableName;
import com.hz.pm.api.common.compare.Compare;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.AllArgsConstructor;
import lombok.Builder;
import lombok.Data;
import lombok.NoArgsConstructor;

import java.io.Serializable;
import java.math.BigDecimal;
import java.time.LocalDateTime;

/**
 * <p>
 * 项目草稿箱
 * </p>
 *
 * @author zpf
 * @since 2023-02-06
 */
@TableName("nd_project_draft")
@ApiModel(value = "NdProjectDraft对象")
@Data
@Builder
@NoArgsConstructor
@AllArgsConstructor
public class ProjectDraft implements Serializable {

    private static final long serialVersionUID = 1L;

    @ApiModelProperty("应用ID")
    @TableId(type = IdType.AUTO)
    private Long id;

    @ApiModelProperty("所属地区编号")
    private String areaCode;

    @ApiModelProperty("所属地区名称")
    private String area;

    @ApiModelProperty("项目阶段")
    private Integer stage;

    @ApiModelProperty("项目状态")
    private Integer status;

    @ApiModelProperty("项目名称")
    private String projectName;

    @ApiModelProperty("历年项目名称")
    private String baseHistorProjName;

    @ApiModelProperty("'历年项目编码'")
    private String baseHistorProjId;

    @ApiModelProperty("历年项目年度")
    private String baseHistorProjYear;


    @ApiModelProperty("是否临时增补 0:否 1:是")
    private Integer isTemporaryAugment;

    @ApiModelProperty("项目负责人")
    private String responsibleMan;

    @ApiModelProperty("负责人手机号码")
    private String responsibleManMobile;

    @ApiModelProperty("项目联系人")
    private String contactName;

    @ApiModelProperty("项目联系人手机号码")
    private String contactPhone;

    @ApiModelProperty("建设单位名称")
    private String buildOrgName;

    @ApiModelProperty("建设单位统一社会信用代码")
    private String orgCreditCode;

    @ApiModelProperty("公司编码code")
    private String buildOrgCode;

    @ApiModelProperty("建设单位浙政钉ID")
    private String buildOrgZheJiangGovDingId;

    @ApiModelProperty("项目类型 01:首次建设 02:迭代升级 03:结转建设 04新运维 05续运维")
    @Compare("项目类型")
    private String projectType;

    @ApiModelProperty("是否省级 1省级 2非省级")
    private Integer baseProvManDeprtType;

    @ApiModelProperty("是否首次新建 0:否 1:是")
    private Integer isFirst;

    @ApiModelProperty("项目预算年度")
    private Integer projectYear;

    @ApiModelProperty("项目建设起始时间")
    private String beginTime;

    @ApiModelProperty("项目建设终止时间")
    private String endTime;

    @ApiModelProperty("四大体系 1:业务应用 2:应用支撑 3:数据资源 4:基础设施")
    private Integer fourSystems;

    @ApiModelProperty("是否数字化改革项目 0:否 1:是")
    private Integer isDigitalReform;

    @ApiModelProperty("综合业务领域")
    private Integer bizDomain;

    @ApiModelProperty("建设层级 P省级 M市级 C县级 T乡镇级 V村级")
    @Compare("建设层级")
    private String buildLevel;

    @ApiModelProperty("贯通层级 P省级 M市级 C县级 T乡镇级 V村级")
    @Compare("贯通层级")
    private String lowestLevel;

    @ApiModelProperty("立项依据")
    private String buildBasis;

    @ApiModelProperty("立项依据说明")
    private String baseBasisEstablish;

    @ApiModelProperty("预算来源说明")
    private String baseBasisAmountOri;

    @ApiModelProperty("发改编码")
    private String developCode;

    @ApiModelProperty("财政编码")
    private String financialCode;

    @ApiModelProperty("是否上云 0:否 1:是")
    private Integer isCloud;

    private String cloudType;

    @ApiModelProperty("等保级别 1:一级 2:二级 3:三级 4:四级 5:五级")
    private Integer protectionLevel;

    @ApiModelProperty("是否密评 0:否 1:是")
    private Integer isSecretComments;

    @ApiModelProperty("项目简介")
    private String projectIntroduction;

    @ApiModelProperty("资金申报情况-申报金额(万元)")
    private BigDecimal declareAmount;

    @ApiModelProperty("资金申报情况-自有金额(万元)")
    private BigDecimal declareHaveAmount;

    @ApiModelProperty("资金申报情况-政府投资-本级财政资金(万元)")
    private BigDecimal declareGovOwnFinanceAmount;

    @ApiModelProperty("资金申报情况-政府投资-上级补助资金(万元)")
    private BigDecimal declareGovSuperiorFinanceAmount;

    @ApiModelProperty("银行贷款(万元)")
    private BigDecimal declareBankLendingAmount;

    @ApiModelProperty("其它资金(万元)")
    private BigDecimal declareOtherAmount;

    @ApiModelProperty("资金分配情况-软件开发(万元)")
    private BigDecimal softwareDevelopmentAmount;

    @ApiModelProperty("资金分配情况-云资源、硬件购置(万元)")
    private BigDecimal cloudHardwarePurchaseAmount;

    @ApiModelProperty("资金分配情况-第三方服务(万元)")
    private BigDecimal thirdPartyAmount;

    @ApiModelProperty("年度支付计划")
    private String paymentPlans;

    private LocalDateTime annualPlanAddTime;

    @ApiModelProperty("核心业务-核心业务模块")
    private String coreBusiness;

    @ApiModelProperty("安全投入-投入项")
    private String safetyInputTitle;

    @ApiModelProperty("安全投入-内容描述")
    private String safetyInputDescribe;

    @ApiModelProperty("安全投入-金额(万元)")
    private BigDecimal safetyInputAmount;

    @ApiModelProperty("附件-初步方案")
    private String preliminaryPlanFile;

    @ApiModelProperty("附件-佐证材料")
    private String supportingMaterialsFile;

    @ApiModelProperty("附件-项目总投资测算明细")
    private String calculationTotalInvestmentFile;

    @ApiModelProperty("附件-申报单位主要职责（单位三定方案）")
    private String mainResponsibilitiesApplicantFile;

    @ApiModelProperty("备注")
    private String projectRemarks;

    @ApiModelProperty("是否包含应用 0:否 1:是")
    private Integer includeApplication;

    @ApiModelProperty("工程形象进度-第一季度")
    private String engineeringSpeedOne;

    @ApiModelProperty("工程形象进度-第二季度")
    private String engineeringSpeedTwo;

    @ApiModelProperty("工程形象进度-第三季度")
    private String engineeringSpeedThree;

    @ApiModelProperty("工程形象进度-第四季度")
    private String engineeringSpeedFour;

    @ApiModelProperty("核心业务-是否开启核心业务模块 false:关闭 true:开启")
    private Boolean isOpenCoreBusiness;

    @ApiModelProperty("安全投入-是否开启安全投入模块 false:关闭 true:开启")
    private Boolean isOpenSafetyInput;

    @ApiModelProperty("工程形象进度-是否开启 false:关闭 true:开启")
    private Boolean isEngineeringSpeed;

    @ApiModelProperty("附件-是否开启 false:关闭 true:开启")
    private Boolean isAccessories;

    @ApiModelProperty("备注-是否开启 false:关闭 true:开启")
    private Boolean isRemarks;

    @ApiModelProperty("年度支付计划-是否开启 false:关闭 true:开启")
    private Boolean isAnnualPlanAmount;

    @ApiModelProperty("一地创新全省共享项目-是否开启 false:关闭 true:开启")
    private Boolean isInnovateWholeProvinceShare;

    private LocalDateTime createOn;

    private LocalDateTime updateOn;

    @ApiModelProperty("流程实例编号")
    private String instCode;

    @ApiModelProperty("安全投入-模块信息")
    private String safetyInputModular;

    @ApiModelProperty("项目申报pdf")
    private String projectPdf;

    @ApiModelProperty("立项申报pdf")
    private String declarationPdf;

    @ApiModelProperty("建设周期（月）")
    private String buildCycle;

    @ApiModelProperty("建设方案文件")
    private String constructionPlanFile;

    @ApiModelProperty("立项批复文件")
    private String approvedFile;

    @ApiModelProperty("批复金额")
    private BigDecimal approvedTotalInvestmentIncrease;

    @ApiModelProperty("是否有上级条线主管部门 0没有 1有")
    private Integer isHigherSuperOrg;

    @ApiModelProperty("上级主管部门")
    private String higherSuperOrg;

    @ApiModelProperty("上级主管部门Code")
    private String higherSuperOrgCode;

    @ApiModelProperty("是否有主管部门 0没有 1有")
    private Integer isSuperOrg;

    @ApiModelProperty("主管部门名称")
    private String superOrg;

    @ApiModelProperty("主管部门Code")
    private String superOrgCode;

    @ApiModelProperty("前端所需验证字段")
    private String allApplicationsDone;

    @ApiModelProperty("项目应用")
    private String projectApplicationList;

    @ApiModelProperty("新增form内容")
    private String dynamicForm;

    @ApiModelProperty("用户id")
    private String userId;

    @ApiModelProperty("项目申报书")
    private String projectApplicationForm;

    @ApiModelProperty("项目编码")
    private String projectCode;

    @ApiModelProperty("可研报告")
    private String baseResearchReportFile;

    @ApiModelProperty("可研批复文件")
    private String baseResearchReportApprovalFile;

    @ApiModelProperty("其他附件")
    private String baseProjOtherFile;

    @ApiModelProperty("项目内容与预期成效")
    private String beseExpectedResults;

    @ApiModelProperty("内容类别")
    private String baseConstructionType;

    @ApiModelProperty("主管单位统一社会信用代码")
    @Compare("主管单位统一社会信用代码")
    private String superOrgCreditCode;

    private Long createBy;
    private Long updateBy;

    @ApiModelProperty("是否开启评审清单")
    private Boolean isReviewChecklist;

    @ApiModelProperty("评审清单")
    private String reviewChecklist;

    @ApiModelProperty("是否推送省里")
    private Boolean push;

    @ApiModelProperty("系统定位")
    private String systemPosition;

    @ApiModelProperty("立项依据")
    private String baseProjBasis;

    @ApiModelProperty("立项依据证明材料")
    private String baseProjBasisFile;

    @ApiModelProperty("暂存类型：1 项目申报、2 立项备案")
    private Integer draftType;

    private String govSystemReplaceInfos;

    @ApiModelProperty("信产项目ID")
    private Long mhProjectId;

    @ApiModelProperty("申报单位领域")
    private Integer unitStrip;

    @ApiModelProperty("评审金额")
    private BigDecimal reviewAmount;

    @ApiModelProperty("批复-自有金额(万元)")
    @Compare("批复-自有金额")
    private BigDecimal approvalHaveAmount;

    @ApiModelProperty("批复-政府投资-本级财政资金(万元)")
    @Compare("批复-政府投资-本级财政资金")
    private BigDecimal approvalGovOwnFinanceAmount;

    @ApiModelProperty("批复-政府投资-上级补助资金(万元)")
    @Compare("批复-政府投资-上级补助资金")
    private BigDecimal approvalGovSuperiorFinanceAmount;

    @ApiModelProperty("立项批复资金(万元)")
    private BigDecimal approvalAmount;

}
