package com.hz.pm.api.projectdeclared.utils;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.hz.pm.api.projectlib.model.entity.Project;
import com.hz.pm.api.projectlib.service.IProjectService;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;

import javax.annotation.PostConstruct;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;
import java.util.stream.Collectors;

/**
 * <p>
 * ProjectIdCodeCacheUtil
 * </p>
 *
 * @author WendyYang
 * @since 18:40 2024/3/25
 */
@Slf4j
@Component
@RequiredArgsConstructor
public class ProjectIdCodeCacheUtil {

    private final IProjectService projectService;

    @PostConstruct
    public void initProjectIdCodeCache() {
        Wrapper<Project> query = Wrappers.lambdaQuery(Project.class)
                .select(Project::getProjectCode, Project::getId);
        for (Project project : projectService.list(query)) {
            put(project.getId(), project.getProjectCode());
        }
        log.info("初始化项目ID编码缓存：{}", CACHE.size());
    }

    //==================================================================================================================

    private static final Map<Long, String> CACHE = new ConcurrentHashMap<>();

    public static String get(Long projectId) {
        return CACHE.get(projectId);
    }

    public static List<Long> get(String projectCode) {
        return CACHE.entrySet()
                .stream()
                .filter(w -> w.getValue().equals(projectCode))
                .map(Map.Entry::getKey)
                .collect(Collectors.toList());
    }

    public static void put(Long projectId, String projectCode) {
        CACHE.put(projectId, projectCode);
    }

    public static Long newest(String projectCode) {
        return CollUtil.max(get(projectCode));
    }

    public static Long newest(Long projectId) {
        String projectCode = get(projectId);
        if (StrUtil.isBlank(projectCode)) {
            return projectId;
        }
        return CollUtil.max(get(projectCode));
    }

}
