package com.hz.pm.api.projectlib.controller;

import com.hz.pm.api.projectdeclared.model.dto.DefaultDeclaredDTO;
import com.hz.pm.api.projectlib.manage.DeclaredRecordManage;
import com.hz.pm.api.projectlib.model.req.DeclaredProjectListReq;
import com.hz.pm.api.projectlib.model.vo.DeclaredProjectListVO;
import com.hz.pm.api.projectlib.model.vo.MhProjectBaseInfoVO;
import com.ningdatech.basic.model.PageVo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.validation.Valid;

/**
 * <p>
 * 前端控制器
 * </p>
 *
 * @author WendyYang
 * @since 2024-01-30
 */
@Api(tags = "信产项目（立项备案）相关接口")
@RestController
@RequiredArgsConstructor
@RequestMapping("/api/v1/declared/record")
public class MhProjectController {

    private final DeclaredRecordManage declaredRecordManage;

    @ApiOperation("立项备案列表")
    @GetMapping("/pageDeclaredProject")
    public PageVo<DeclaredProjectListVO> pageDeclaredProject(DeclaredProjectListReq req) {
        return declaredRecordManage.pageDeclaredProject(req);
    }

    @ApiOperation("信产项目基本信息")
    @GetMapping("/baseInfo/{mhProjectId}")
    public MhProjectBaseInfoVO projectBaseInfo(@PathVariable Long mhProjectId) {
        return declaredRecordManage.getProjectBaseInfo(mhProjectId);
    }

    @ApiOperation("立项备案提交")
    @PostMapping("/submit")
    public void submit(@Valid @RequestBody DefaultDeclaredDTO req) {
        declaredRecordManage.startDeclaredRecord(req);
    }

    @ApiOperation("立项备案重新提交")
    @PostMapping("/resubmit")
    public void resubmit(@Valid @RequestBody DefaultDeclaredDTO req) {
        declaredRecordManage.restartDeclaredRecord(req);
    }

    @ApiOperation("初始化信产项目至项目库")
    @GetMapping("/initProjectLib")
    public void initMhProjectToProjectLib() {
        declaredRecordManage.initMhProjectToProject();
    }

    @GetMapping("/importReplaceSystemInfos")
    public void importReplaceSystemInfos(MultipartFile file) {
        declaredRecordManage.importReplaceSystemInfos(file);
    }

    @GetMapping("/importProjectFinanceInfos")
    public void importProjectFinanceInfos(MultipartFile file) {
        declaredRecordManage.importProjectFinanceInfos(file);
    }

}
