package com.hz.pm.api.projectlib.handle.stage;

import com.hz.pm.api.common.statemachine.event.ProjectStateChangeEvent;
import com.hz.pm.api.projectdeclared.model.entity.Purchase;
import com.hz.pm.api.projectdeclared.model.entity.PurchaseInst;
import com.hz.pm.api.projectdeclared.service.IPurchaseInstService;
import com.hz.pm.api.projectdeclared.service.IPurchaseService;
import com.hz.pm.api.projectlib.handle.AbstractProcessHandle;
import com.hz.pm.api.projectlib.handle.HandleParamContext;
import com.hz.pm.api.projectlib.model.entity.Project;
import com.hz.pm.api.projectlib.model.entity.ProjectStatusChange;
import com.hz.pm.api.projectlib.service.IProjectStatusChangeService;
import com.wflow.workflow.bean.vo.ProcessDetailVO;
import com.wflow.workflow.enums.StepStatusEnum;
import lombok.RequiredArgsConstructor;
import org.springframework.stereotype.Component;

import java.util.List;

import static com.hz.pm.api.common.statemachine.event.ProjectStateChangeEvent.SUBMIT_OPERATION_PLAN;
import static com.hz.pm.api.projectlib.model.enumeration.status.ProjectStatus.ON_PURCHASING;
import static com.hz.pm.api.projectlib.model.enumeration.status.ProjectStatus.TO_BE_PURCHASED;

/**
 * 招标采购处理
 *
 * @author CMM
 * @since 2023/02/24 14:35
 */
@Component
@RequiredArgsConstructor
public class ProjectPurchaseHandle extends AbstractProcessHandle {

    private final IProjectStatusChangeService projectStatusChangeService;
    private final IPurchaseService purchaseService;
    private final IPurchaseInstService purchaseInstService;

    @Override
    public int getOrder() {
        return getProcessNode().getOrder();
    }

    @Override
    public ProcessNode getProcessNode() {
        return ProcessNode.PROJECT_PURCHASE;
    }

    @Override
    public void handle(HandleParamContext ctx) {
        ProcessDetailVO detail = new ProcessDetailVO();
        detail.setProcessName(getProcessNode().getName());
        ctx.getProcessDetails().add(detail);
        if (!StepStatusEnum.COMPLETED.equals(ctx.getPreStepStatus())) {
            detail.setStepStatus(StepStatusEnum.NOT_START);
            return;
        }
        Project project = ctx.getProject();
        if (TO_BE_PURCHASED.eq(project.getStatus())) {
            detail.setStepStatus(StepStatusEnum.NOT_START);
        } else if (ON_PURCHASING.eq(project.getStatus())) {
            detail.setStepStatus(StepStatusEnum.ON_GOING);
        } else {
            // 根据采购备案的时间获取
            ProjectStatusChange change = ctx.getProjectStateChange(SUBMIT_OPERATION_PLAN);
            if (change == null) {
                detail.setStepStatus(StepStatusEnum.NOT_START);
            } else {
                detail.setStepStatus(StepStatusEnum.COMPLETED);
                detail.setFinishTime(change.getCreateOn());
                List<Purchase> purchases = purchaseService.listByProjectId(project.getId());
                ctx.setPurchases(purchases);
                List<PurchaseInst> purchaseInstList = purchaseInstService.listByProjectId(project.getId());
                ctx.setPurchaseInstList(purchaseInstList);
            }
        }
        ctx.setPreStepStatus(detail.getStepStatus());
    }

}
