package com.hz.pm.api.projectlib.manage;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.lang.Assert;
import cn.hutool.core.map.MapUtil;
import cn.hutool.core.util.StrUtil;
import cn.hutool.poi.excel.ExcelUtil;
import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.hz.pm.api.common.enumeration.CommonEnum;
import com.hz.pm.api.common.enumeration.ProjectProcessType;
import com.hz.pm.api.common.helper.UserInfoHelper;
import com.hz.pm.api.common.model.constant.TypeReferenceConst;
import com.hz.pm.api.datascope.model.DataScopeDTO;
import com.hz.pm.api.datascope.utils.DataScopeUtil;
import com.hz.pm.api.external.model.enumeration.MhUnitStripEnum;
import com.hz.pm.api.external.model.enumeration.MhUnitTypeEnum;
import com.hz.pm.api.external.todo.MHTodoClient;
import com.hz.pm.api.external.todo.enumerization.MHTodoTypeEnum;
import com.hz.pm.api.projectdeclared.manage.DeclaredProjectHelper;
import com.hz.pm.api.projectdeclared.model.dto.DefaultDeclaredDTO;
import com.hz.pm.api.projectdeclared.model.dto.ProjectConditionDTO;
import com.hz.pm.api.projectdeclared.service.IProjectDraftService;
import com.hz.pm.api.projectdeclared.utils.ProjectCodeGenUtil;
import com.hz.pm.api.projectlib.entity.ProjectGovSystemReplaceInfos;
import com.hz.pm.api.projectlib.model.dto.ImportProjectFinanceInfosDTO;
import com.hz.pm.api.projectlib.model.dto.ImportReplaceSystemInfosDTO;
import com.hz.pm.api.projectlib.model.dto.ProjectDTO;
import com.hz.pm.api.projectlib.model.entity.MhProject;
import com.hz.pm.api.projectlib.model.entity.MhProjectSchemaTargetData;
import com.hz.pm.api.projectlib.model.entity.Project;
import com.hz.pm.api.projectlib.model.enumeration.GovSystemReplaceTypeEnum;
import com.hz.pm.api.projectlib.model.enumeration.status.ProjectStatus;
import com.hz.pm.api.projectlib.model.req.DeclaredProjectListReq;
import com.hz.pm.api.projectlib.model.vo.DeclaredProjectListVO;
import com.hz.pm.api.projectlib.model.vo.MhProjectBaseInfoVO;
import com.hz.pm.api.projectlib.service.IMhProjectSchemaTargetDataService;
import com.hz.pm.api.projectlib.service.IMhProjectService;
import com.hz.pm.api.projectlib.service.IProjectGovSystemReplaceInfosService;
import com.hz.pm.api.projectlib.service.IProjectService;
import com.hz.pm.api.staging.enums.MsgTypeEnum;
import com.hz.pm.api.sys.manage.NoticeManage;
import com.hz.pm.api.sys.manage.ProcessModelManage;
import com.hz.pm.api.todocenter.constant.WorkNoticeConst;
import com.hz.pm.api.user.helper.MhUnitCache;
import com.hz.pm.api.user.model.dto.UnitDTO;
import com.hz.pm.api.user.model.entity.MhUnit;
import com.hz.pm.api.user.security.model.UserFullInfoDTO;
import com.hz.pm.api.user.security.model.UserInfoDetails;
import com.hz.pm.api.user.service.IMhUnitService;
import com.hz.pm.api.user.util.LoginUserUtil;
import com.ningdatech.basic.exception.BizException;
import com.ningdatech.basic.function.VUtils;
import com.ningdatech.basic.model.PageVo;
import com.ningdatech.basic.util.CollUtils;
import com.wflow.bean.entity.WflowModels;
import com.wflow.workflow.bean.dto.OrgInfoDTO;
import com.wflow.workflow.bean.vo.ProcessStartParamsVo;
import com.wflow.workflow.service.ProcessInstanceService;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.multipart.MultipartFile;

import java.io.InputStream;
import java.math.BigDecimal;
import java.time.LocalDateTime;
import java.util.*;
import java.util.concurrent.atomic.AtomicInteger;
import java.util.stream.Collectors;

/**
 * <p>
 * MhProjectManage
 * </p>
 *
 * @author WendyYang
 * @since 19:06 2024/1/30
 */
@Slf4j
@Component
@RequiredArgsConstructor
public class DeclaredRecordManage {

    private final UserInfoHelper userInfoHelper;
    private final ProjectCodeGenUtil projectCodeGenUtil;
    private final IMhProjectService mhProjectService;
    private final IMhProjectSchemaTargetDataService schemaTargetDataService;
    private final IProjectService projectService;
    private final MhUnitCache mhUnitCache;
    private final IMhUnitService mhUnitService;
    private final NoticeManage noticeManage;
    private final IProjectDraftService projectDraftService;
    private final ProcessModelManage processModelManage;
    private final DeclaredProjectHelper declaredProjectHelper;
    private final ProcessInstanceService processInstanceService;
    private final ProjectLibManage projectLibManage;
    private final IProjectGovSystemReplaceInfosService systemReplaceInfosService;
    private final MHTodoClient mhTodoClient;

    private LambdaQueryWrapper<Project> buildQuery(DeclaredProjectListReq req) {
        LambdaQueryWrapper<Project> query = Wrappers.lambdaQuery(Project.class)
                .like(StrUtil.isNotBlank(req.getProjectName()), Project::getProjectName, req.getProjectName())
                .like(StrUtil.isNotBlank(req.getDeclaredUnit()), Project::getBuildOrgName, req.getDeclaredUnit())
                .eq(req.getUnitStrip() != null, Project::getUnitStrip, req.getUnitStrip());
        if (req.getCreateDateMin() != null) {
            query.ge(Project::getCreateOn, req.getCreateDateMin());
        }
        if (req.getCreateDateMax() != null) {
            query.lt(Project::getCreateOn, req.getCreateDateMax().minusDays(1));
        }
        return query;
    }

    private boolean buildProjectLibPermission(LambdaQueryWrapper<Project> query, UserFullInfoDTO user, Long declaredUnitId) {
        boolean queryState = true;
        Optional<DataScopeDTO> currentUserDataScope = DataScopeUtil.getCurrentUserDataScopeHasUserId(user);
        if (!currentUserDataScope.isPresent()) {
            log.warn("没有取到权限信息 当前查询 没有权限条件");
            queryState = false;
        } else {
            List<Long> viewUnitIds = new ArrayList<>();
            switch (currentUserDataScope.get().getRole()) {
                case NORMAL_MEMBER:
                    //普通用户 只能看到自己单位去申报的
                    viewUnitIds.add(user.getMhUnitId());
                    break;
                case COMPANY_MANAGER:
                    List<Long> childUnitIds = mhUnitCache.getViewChildIdsRecursion(user.getMhUnitId());
                    if (declaredUnitId != null && childUnitIds.contains(declaredUnitId)
                            && !declaredUnitId.equals(user.getMhUnitId())) {
                        childUnitIds = mhUnitCache.getViewChildIdsRecursion(declaredUnitId);
                    }
                    viewUnitIds.addAll(childUnitIds);
                    //单位管理员 看到自己单位去申报的 + 待预审的主管单位是自己单位的项目
                    break;
                case SUPER_ADMIN:
                    if (declaredUnitId != null) {
                        viewUnitIds.addAll(mhUnitCache.getViewChildIdsRecursion(declaredUnitId));
                    }
                    //超级管理员 看到丽水全市的 并且也要判断他 同时是不是单位管理员
                    break;
                case VISITOR:
                    //访客可以看全市的
                    break;
                case DASHBOARD:
                    break;
                default:
                    //没有权限的话 就让它查不到
                    queryState = false;
                    break;
            }
            if (queryState && !viewUnitIds.isEmpty()) {
                query.in(Project::getBuildOrgCode, CollUtils.convert(viewUnitIds, String::valueOf));
            }
        }
        return queryState;
    }

    public PageVo<DeclaredProjectListVO> pageDeclaredProject(DeclaredProjectListReq req) {
        LambdaQueryWrapper<Project> query = buildQuery(req);
        UserFullInfoDTO user = userInfoHelper.getUserFullInfo(LoginUserUtil.getUserId());
        // 查询审核中、失败、待立项备案的项目信息
        if (!buildProjectLibPermission(query, user, req.getDeclaredUnitId())) {
            return PageVo.empty();
        }
        query.in(Project::getStatus, ProjectStatus.DECLARED_APPROVED_RECORD_FAILED.getCode(),
                        ProjectStatus.DECLARED_APPROVED_TO_BE_RECORD.getCode(),
                        ProjectStatus.DECLARED_APPROVED_RECORD_AUDITING.getCode())
                .eq(Project::getNewest, Boolean.TRUE)
                .orderByDesc(Project::getUpdateOn);
        Page<Project> page = projectService.page(req.page(), query);
        if (page.getTotal() == 0) {
            return PageVo.empty();
        }
        List<String> projCodes = CollUtils.fieldList(page.getRecords(), Project::getProjectCode);
        List<MhProject> mhProjList = mhProjectService.listByProjectCode(projCodes);
        Map<String, MhProject> mhProjMap = CollUtils.listToMap(mhProjList, MhProject::getProjectCode);
        List<DeclaredProjectListVO> data = page.getRecords().stream().map(w -> {
            DeclaredProjectListVO item = new DeclaredProjectListVO();
            MhProject mhProj = mhProjMap.get(w.getProjectCode());
            if (mhProj != null) {
                item.setMhProjectId(mhProj.getId());
            }
            item.setStatus(w.getStatus());
            item.setStage(w.getStage());
            item.setProjectId(w.getId());
            item.setUnitStrip(w.getUnitStrip());
            item.setUnitStripName(MhUnitStripEnum.getVal(w.getUnitStrip()));
            item.setDeclaredUnit(w.getBuildOrgName());
            item.setProjectCode(w.getProjectCode());
            item.setProjectName(w.getProjectName());
            item.setDeclaredAmount(w.getReviewAmount());
            item.setCreateTime(w.getCreateOn());
            return item;
        }).collect(Collectors.toList());
        return PageVo.of(data, page.getTotal());
    }

    public MhProjectBaseInfoVO getProjectBaseInfo(Long mhProjectId) {
        MhProject mhProject = mhProjectService.getById(mhProjectId);
        if (mhProject == null) {
            throw BizException.wrap("项目不存在");
        }
        MhProjectBaseInfoVO detail = new MhProjectBaseInfoVO();
        detail.setMhProjectId(mhProject.getId());
        detail.setProjectName(mhProject.getProjectName());
        detail.setDeclaredUnit(mhProject.getUnitName());
        detail.setDeclaredUnitId(mhProject.getUnitId());
        detail.setTotalInvestAmount(mhProject.getTotalMoney());
        detail.setReviewInvestAmount(mhProject.getMoney());
        detail.setProjectCode(mhProject.getProjectCode());
        if (StrUtil.isNotBlank(detail.getProjectCode())) {
            Optional<Long> projectId = projectService.getNewProjectId(detail.getProjectCode());
            projectId.ifPresent(detail::setProjectId);
        }
        MhProjectSchemaTargetData schemaTargetData = schemaTargetDataService.getByMhProjectId(mhProjectId);
        if (schemaTargetData != null) {
            detail.setTotalInvestBudgetAmount(schemaTargetData.getMoney());
        }
        UnitDTO unit = mhUnitCache.getById(mhProject.getId());
        if (unit != null) {
            detail.setDeclaredUnitUnifiedSocialCreditCode(unit.getUnifiedSocialCreditCode());
        }
        return detail;
    }

    @Transactional(rollbackFor = Exception.class)
    public synchronized void startDeclaredRecord(DefaultDeclaredDTO req) {
        ProjectDTO reqProj = req.getProjectInfo();
        UserInfoDetails user = LoginUserUtil.loginUserDetail();

        reqProj.setBuildOrgCode(user.getMhUnitIdStr());
        reqProj.setBuildOrgName(user.getMhUnitName());

        Project oldProj = projectService.getNoNull(reqProj.getId());

        VUtils.isTrue(!ProjectStatus.DECLARED_APPROVED_TO_BE_RECORD.eq(oldProj.getStatus()))
                .throwMessage("提交失败，该项目不是立项备案审核不通过状态或者未立项阶段");

        // 项目名称去重
        declaredProjectHelper.projectNameDuplicateCheck(reqProj);

        // 写死 是否有主管单位
        reqProj.setIsHigherSuperOrg(CommonEnum.YES.getCode());
        reqProj.setIsSuperOrg(CommonEnum.YES.getCode());

        //如果主管单位没有 那么主管单位就是自己
        if (CommonEnum.NO.getCode().equals(reqProj.getIsSuperOrg())) {
            reqProj.setSuperOrgCode(user.getMhUnitIdStr());
            reqProj.setSuperOrg(user.getMhUnitName());
        }

        ProjectProcessType instType = ProjectProcessType.PROJECT_RECORD_APPROVAL_PROCESS;
        WflowModels model = processModelManage.getWflowModelsNoNull(instType, user.getMhUnitId());

        ProcessStartParamsVo params = new ProcessStartParamsVo();
        params.setUser(declaredProjectHelper.buildUser(user));
        params.setProcessUsers(Collections.emptyMap());
        //放入条件判断的项目字段
        ProjectConditionDTO condition = new ProjectConditionDTO();
        BeanUtils.copyProperties(req.getProjectInfo(), condition);
        req.getFormData().putAll(JSON.parseObject(JSON.toJSONString(condition), TypeReferenceConst.STR_OBJ_MAP));
        params.setFormData(req.getFormData());
        //开始申报
        // 获取发起单位、发起单位主管单位、发起单位上级主管条线单位信息
        Project project = new Project();
        BeanUtils.copyProperties(reqProj, project);
        Map<String, OrgInfoDTO> orgModelMap = declaredProjectHelper.buildOrgModelMap(user, project);
        String instanceId = processInstanceService.startProcessLs(model, params, orgModelMap);
        log.info("立项备案成功 【{}】", instanceId);

        // 如果是重新提交的话 判断下 项目是否存在
        // 保存项目相关
        Project newProj = projectLibManage.saveProjectInDeclared(reqProj, instanceId, user.getUserId(),
                ProjectStatus.NOT_APPROVED, ProjectStatus.DECLARED_APPROVED_TO_BE_RECORD,
                ProjectProcessType.PROJECT_RECORD_APPROVAL_PROCESS);
        if (Objects.nonNull(reqProj.getDraftId())) {
            //如果是草稿箱提交 删除对应的草稿箱
            projectDraftService.removeById(reqProj.getDraftId());
        }
        if (reqProj.getMhProjectId() != null) {
            // 申报项目完成后 保存项目编码
            Wrapper<MhProject> update = Wrappers.lambdaUpdate(MhProject.class)
                    .set(MhProject::getProjectCode, newProj.getProjectCode())
                    .eq(MhProject::getId, reqProj.getMhProjectId());
            mhProjectService.update(update);
        }
        //发送给第一个审批人消息
        noticeManage.sendFirstUser(newProj, model.getFormName(), instanceId,
                WorkNoticeConst.PASS_MSG_TEMPLATE, MsgTypeEnum.PROJECT_REVIEW);
        // 完成立项备案待办
        mhTodoClient.completeTodo(MHTodoTypeEnum.DECLARED_RECORD_APPLY, newProj.getProjectCode());
    }

    @Transactional(rollbackFor = Exception.class)
    public synchronized void restartDeclaredRecord(DefaultDeclaredDTO req) {
        UserInfoDetails user = LoginUserUtil.loginUserDetail();
        ProjectDTO reqProj = req.getProjectInfo();
        Assert.notNull(reqProj.getId(), "提交失败，缺少项目ID!");
        reqProj.setBuildOrgCode(user.getMhUnitIdStr());
        reqProj.setBuildOrgName(user.getMhUnitName());

        Project oldProj = projectService.getNoNull(reqProj.getId());

        VUtils.isTrue(!ProjectStatus.DECLARED_APPROVED_RECORD_FAILED.eq(oldProj.getStatus()))
                .throwMessage("提交失败，该项目不是立项备案审核不通过状态或者未立项阶段");

        //项目名称去重
        if (StrUtil.isNotBlank(reqProj.getProjectName()) &&
                !reqProj.getProjectName().equals(oldProj.getProjectName())) {
            reqProj.setProjectCode(oldProj.getProjectCode());
            declaredProjectHelper.projectNameDuplicateCheck(reqProj);
        }
        UnitDTO unit = mhUnitCache.getById(user.getMhUnitId());
        Assert.notNull(unit, "申报单位不存在");
        MhUnitTypeEnum unitType = Objects.requireNonNull(unit.getType(), "申报单位类型错误");
        reqProj.setUnitStrip(unitType.getStrip().getCode());
        // 写死 是否有主管单位
        oldProj.setIsHigherSuperOrg(CommonEnum.YES.getCode());
        oldProj.setIsSuperOrg(CommonEnum.YES.getCode());

        //如果主管单位没有 那么主管单位就是自己
        if (CommonEnum.NO.eq(reqProj.getIsSuperOrg())) {
            oldProj.setSuperOrgCode(user.getMhUnitIdStr());
            oldProj.setSuperOrg(user.getMhUnitName());
            reqProj.setSuperOrgCode(user.getMhUnitIdStr());
            reqProj.setSuperOrg(user.getMhUnitName());
        }

        ProjectProcessType instType = ProjectProcessType.PROJECT_RECORD_APPROVAL_PROCESS;
        WflowModels model = processModelManage.getWflowModelsNoNull(instType, user.getMhUnitId());

        ProcessStartParamsVo params = new ProcessStartParamsVo();
        params.setUser(declaredProjectHelper.buildUser(user));
        params.setProcessUsers(Collections.emptyMap());

        //把条件值给放入工作流
        declaredProjectHelper.buildCondition(params, req);

        Project project = new Project();
        BeanUtils.copyProperties(reqProj, project);
        Map<String, OrgInfoDTO> orgModelMap = declaredProjectHelper.buildOrgModelMap(user, project);
        String instId = processInstanceService.startProcessLs(model, params, orgModelMap);

        //保存项目相关
        Project newProj = projectLibManage.saveProjectInDeclared(reqProj, instId, user.getUserId(),
                ProjectStatus.NOT_APPROVED,
                ProjectStatus.DECLARED_APPROVED_RECORD_FAILED,
                instType);
        //发送给第一个审批人消息
        noticeManage.sendFirstUser(newProj, model.getFormName(), instId,
                WorkNoticeConst.PASS_MSG_TEMPLATE, MsgTypeEnum.PROJECT_REVIEW);
    }

    @Transactional(rollbackFor = Exception.class)
    public synchronized void initMhProjectToProject() {
        Wrapper<MhProject> query = Wrappers.lambdaQuery(MhProject.class)
                .isNull(MhProject::getProjectCode);
        List<MhProject> mhProjects = mhProjectService.list(query);
        if (mhProjects.isEmpty()) {
            return;
        }
        for (MhProject mhProj : mhProjects) {
            if (StrUtil.isNotBlank(mhProj.getProjectCode())) {
                continue;
            }
            LocalDateTime now = LocalDateTime.now();
            Project project = new Project();
            project.setProjectName(mhProj.getProjectName());
            project.setBuildOrgCode(String.valueOf(mhProj.getUnitId()));
            project.setBuildOrgName(mhProj.getUnitName());
            project.setUnitStrip(mhProj.getUnitStrip());
            project.setVersion(0);
            project.setNewest(Boolean.TRUE);
            project.setStatus(ProjectStatus.DECLARED_APPROVED_TO_BE_RECORD.getCode());
            project.setStage(ProjectStatus.NOT_APPROVED.getCode());
            ProjectDTO tmpProj = BeanUtil.copyProperties(project, ProjectDTO.class);
            String projectCode = projectCodeGenUtil.generateProjectCode(tmpProj);
            project.setProjectCode(projectCode);
            project.setCreateOn(now);
            project.setCreateBy(0L);
            project.setUpdateOn(now);
            project.setUpdateBy(0L);
            projectService.save(project);
            mhProjectService.update(Wrappers.lambdaUpdate(MhProject.class)
                    .set(MhProject::getProjectCode, projectCode)
                    .eq(MhProject::getId, mhProj.getId()));
        }
    }

    @Transactional(rollbackFor = Exception.class)
    public void importProjectFinanceInfos(MultipartFile file) {
        try (InputStream is = file.getInputStream()) {
            List<Map<String, Object>> rows = ExcelUtil.getReader(is).readAll();
            if (rows.isEmpty()) {
                return;
            }
            List<ImportProjectFinanceInfosDTO> importDataList = CollUtils.fieldList(rows, w -> {
                String projectName = MapUtil.getStr(w, "项目名称");
                Assert.notBlank(projectName, "项目名称不能为空");
                ImportProjectFinanceInfosDTO financeInfo = new ImportProjectFinanceInfosDTO();
                financeInfo.setProjectName(projectName);
                financeInfo.setUnitName(MapUtil.getStr(w, "单位名称"));
                Assert.notBlank(financeInfo.getUnitName(), "单位名称不能为空");
                financeInfo.setDeclareAmount(MapUtil.get(w, "合计", BigDecimal.class));
                financeInfo.setDeclareHaveAmount(MapUtil.get(w, "其他资金", BigDecimal.class));
                financeInfo.setDeclareGovOwnFinanceAmount(MapUtil.get(w, "市本级信息化专项资金", BigDecimal.class));
                financeInfo.setDeclareGovSuperiorFinanceAmount(MapUtil.get(w, "上级补助资金", BigDecimal.class));
                return financeInfo;
            });
            for (ImportProjectFinanceInfosDTO importData : importDataList) {
                String projectName = importData.getProjectName();
                Project project = projectService.getByProjectName(projectName);
                if (project == null) {
                    List<MhUnit> units = mhUnitService.listByName(importData.getUnitName());
                    if (units.size() != 1) {
                        continue;
                    }
                    MhUnit unit = units.get(0);
                    MhUnitTypeEnum unitType = MhUnitTypeEnum.getByCode(unit.getType()).get();
                    MhProject mhProject = new MhProject();
                    mhProject.setUnitStrip(unitType.getStrip().getCode());
                    mhProject.setUnitName(unit.getName());
                    mhProject.setUnitId(unit.getId());
                    mhProject.setCreateTime(LocalDateTime.now());
                    mhProject.setProjectName(importData.getProjectName());
                    LocalDateTime now = LocalDateTime.now();
                    mhProject.setProjectCode(projectCodeGenUtil.generateProjectCode(now.getYear()));
                    mhProjectService.save(mhProject);
                    project = new Project();
                    project.setProjectCode(mhProject.getProjectCode());
                    project.setNewest(Boolean.TRUE);
                    project.setVersion(1);
                    project.setStage(ProjectStatus.NOT_APPROVED.getCode());
                    project.setStatus(ProjectStatus.DECLARED_APPROVED_TO_BE_RECORD.getCode());
                    project.setCreateOn(now);
                    project.setUpdateOn(now);
                    project.setUnitStrip(mhProject.getUnitStrip());
                    project.setBuildOrgName(unit.getName());
                    project.setBuildOrgCode(String.valueOf(unit.getId()));
                }
                project.setApprovalAmount(importData.getDeclareAmount());
                project.setApprovalGovOwnFinanceAmount(importData.getDeclareGovOwnFinanceAmount());
                project.setApprovalGovSuperiorFinanceAmount(importData.getDeclareGovSuperiorFinanceAmount());
                project.setApprovalHaveAmount(importData.getDeclareHaveAmount());
                projectService.saveOrUpdate(project);
            }
        } catch (Exception e) {
            log.info("读取数据失败：", e);
            throw new BizException(e);
        }
    }

    @Transactional(rollbackFor = Exception.class)
    public void importReplaceSystemInfos(MultipartFile file) {
        try (InputStream is = file.getInputStream()) {
            List<Map<String, Object>> rows = ExcelUtil.getReader(is)
                    .readAll();
            if (rows.isEmpty()) {
                return;
            }
            List<ImportReplaceSystemInfosDTO> importDataList = CollUtils.convert(rows, w -> {
                ImportReplaceSystemInfosDTO importData = new ImportReplaceSystemInfosDTO();
                importData.setProjectName(MapUtil.getStr(w, "项目名称"));
                Assert.notBlank(importData.getProjectName(), "项目名称不能为空");
                importData.setReplaceType(MapUtil.getStr(w, "替代方式"));
                importData.setSourceName(MapUtil.getStr(w, "替代前系统"));
                importData.setTargetName(MapUtil.getStr(w, "替代后系统"));
                importData.setUnitName(MapUtil.getStr(w, "单位"));
                if (StrUtil.equals(importData.getReplaceType(), "/")
                        || StrUtil.isBlank(importData.getReplaceType())) {
                    importData.setReplaceType(null);
                } else {
                    GovSystemReplaceTypeEnum typeEnum = GovSystemReplaceTypeEnum.getByVal(importData.getReplaceType())
                            .orElseThrow(() -> BizException.wrap("替代方式无效：%s", importData.getReplaceType()));
                    importData.setReplaceTypeCode(typeEnum.getCode());
                }
                return importData;
            });
            Map<String, List<ImportReplaceSystemInfosDTO>> groupByProject = CollUtils.group(importDataList,
                    ImportReplaceSystemInfosDTO::getProjectName);
            List<Long> projectIds = new ArrayList<>();
            List<ProjectGovSystemReplaceInfos> replaceInfos = new ArrayList<>();
            for (Map.Entry<String, List<ImportReplaceSystemInfosDTO>> entry : groupByProject.entrySet()) {
                Project project = projectService.getByProjectName(entry.getKey());
                if (project == null) {
                    continue;
                }
                projectIds.add(project.getId());
                Map<String, Integer> batchNoMap = new HashMap<>();
                AtomicInteger integer = new AtomicInteger(1);
                for (ImportReplaceSystemInfosDTO w : entry.getValue()) {
                    Integer batchNo = batchNoMap.computeIfAbsent(w.getTargetName(),
                            k -> integer.getAndIncrement());
                    ProjectGovSystemReplaceInfos replaceInfo = new ProjectGovSystemReplaceInfos();
                    replaceInfo.setProjectId(project.getId());
                    Integer typeCode = w.getReplaceTypeCode();
                    replaceInfo.setReplaceType(typeCode == null ? "/" : typeCode.toString());
                    replaceInfo.setBatchNo(batchNo);
                    replaceInfo.setSourceSystem(w.getSourceName());
                    replaceInfo.setTargetSystem(w.getTargetName());
                    replaceInfos.add(replaceInfo);
                }
            }
            if (replaceInfos.isEmpty()) {
                return;
            }
            systemReplaceInfosService.removeByProjectIds(projectIds);
            systemReplaceInfosService.saveBatch(replaceInfos);
        } catch (Exception e) {
            log.info("读取数据失败：", e);
            throw new BizException(e);
        }
    }


}
