package com.hz.pm.api.projectlib.model.enumeration.status;

import cn.hutool.core.util.StrUtil;
import com.ningdatech.basic.exception.BizException;
import lombok.AllArgsConstructor;
import lombok.Getter;

import java.util.Arrays;
import java.util.Optional;

/**
 * <p>
 * ProjectStatusEnum
 * </p>
 *
 * @author Poffy
 * @since 16:54 2023/2/11
 */
@Getter
@AllArgsConstructor
public enum ProjectStatus implements IStatus<Integer, String> {

    /**
     * 项目阶段：未立项
     */
    NOT_APPROVED(10000, "未立项"),
    TO_BE_DECLARED(10015, "待申报"),

    /**
     * 合规性审查
     */
    ON_COMPLIANCE_REVIEW(10030, "合规性审查中"),
    COMPLIANCE_REVIEW_FAILED(10031, "合规性审查不通过"),

    /**
     * 项目评审
     */
    WITHOUT_PROJECT_REVIEW(10040, "待项目评审"),
    ON_PROJECT_REVIEW(10041, "项目评审中"),
    PROJECT_REVIEW_FAILED(10042, "项目评审不通过"),

    ON_ANNUAL_PLAN(10010, "年度计划中"),

    /**
     * 信产项目备案
     */
    DECLARED_APPROVED_TO_BE_RECORD(10022, "待立项备案"),
    DECLARED_APPROVED_RECORD_AUDITING(10023, "立项备案审核中"),
    DECLARED_APPROVED_RECORD_FAILED(10024, "立项备案审核不通过"),

    /**
     * 项目阶段：已立项
     */
    PROJECT_APPROVED(20000, "已立项"),
    TO_BE_PURCHASED(20001, "待采购"),
    ON_PURCHASING(20007, "采购中"),

    TO_BE_FIRST_INSPECTED(20008, "待上传初验材料"),
    ON_PILOT_RUNNING(20009, "试运行中"),

    ON_FINALLY_INSPECTED(20004, "终验审核中"),
    FINALLY_INSPECTED_FAILED(20005, "终验审核不通过"),

    /**
     * 项目阶段：已归档
     */
    ARCHIVED(30000, "已归档"),
    ACCEPTED(30001, "已验收");

    private final Integer code;
    private final String desc;

    public static String getDesc(Integer code) {
        return get(code).flatMap(w -> Optional.of(w.getDesc())).orElse(StrUtil.EMPTY);
    }

    private static Optional<ProjectStatus> get(Integer code) {
        return Arrays.stream(values()).filter(w -> w.getCode().equals(code)).findFirst();
    }

    public static ProjectStatus getNoNull(Integer code) {
        return get(code).orElseThrow(() -> BizException.wrap("项目状态无效：%s", code));
    }

}
