package com.hz.pm.api.projectlib.model.enumeration.status;

import cn.hutool.core.util.StrUtil;
import com.ningdatech.basic.exception.BizException;
import lombok.AllArgsConstructor;
import lombok.Getter;

import java.util.Arrays;
import java.util.Optional;

/**
 * <p>
 * SelfTestStatusEnum
 * </p>
 *
 * @author WendyYang
 * @since 10:16 2024/3/27
 */
@Getter
@AllArgsConstructor
public enum TenderAdaptStatus implements IStatus<Integer, String> {

    WITHOUT_ADAPT_INFO(2100, "待填写适配改造信息"),
    ADAPT_INFO_AUDIT(2101, "适配改造信息审核中"),
    ADAPT_INFO_FAILED(2102, "适配改造信息审核失败"),
    ADAPT_INFO_PASSED(2103, "适配改造信息审核通过");

    private final Integer code;
    private final String desc;

    private static Optional<TenderAdaptStatus> get(Integer code) {
        return Arrays.stream(values()).filter(w -> w.eq(code)).findFirst();
    }

    public static TenderAdaptStatus getNoNull(Integer code) {
        return get(code).orElseThrow(() -> BizException.wrap("无效的适配改造状态：%s", code));
    }

    public static String getDesc(Integer code) {
        return get(code).flatMap(w -> Optional.of(w.getDesc())).orElse(StrUtil.EMPTY);
    }

}
