package com.hz.pm.api.projectlib.model.enumeration.status;

import cn.hutool.core.util.StrUtil;
import com.ningdatech.basic.exception.BizException;
import lombok.AllArgsConstructor;
import lombok.Getter;

import java.util.Arrays;
import java.util.Optional;

/**
 * <p>
 * SelfTestStatusEnum
 * </p>
 *
 * @author WendyYang
 * @since 10:16 2024/3/27
 */
@Getter
@AllArgsConstructor
public enum TenderSelfTestStatus implements IStatus<Integer, String> {

    WITHOUT_SELF_TEST_INFO(1100, "待填写系统自测信息"),
    SELF_TEST_INFO_AUDIT(1101, "系统自测信息审核中"),
    SELF_TEST_INFO_FAILED(1102, "系统自测信息审核失败"),
    SELF_TEST_INFO_PASSED(1103, "系统自测信息审核通过");

    private final Integer code;
    private final String desc;

    private static Optional<TenderSelfTestStatus> get(Integer code) {
        return Arrays.stream(values())
                .filter(w -> w.eq(code))
                .findFirst();
    }

    public static TenderSelfTestStatus getNoNull(Integer code) {
        return get(code).orElseThrow(() -> BizException.wrap("无效的系统自测状态：%s", code));
    }

    public static String getDesc(Integer code) {
        return get(code).flatMap(w -> Optional.of(w.getDesc())).orElse(StrUtil.EMPTY);
    }

}
