package com.hz.pm.api.projectlib.model.enumeration.status;

import cn.hutool.core.util.StrUtil;
import com.ningdatech.basic.exception.BizException;
import lombok.AllArgsConstructor;
import lombok.Getter;

import java.util.Arrays;
import java.util.Optional;

/**
 * <p>
 * 标段状态
 * </p>
 *
 * @author WendyYang
 * @since 15:49 2024/2/23
 */
@Getter
@AllArgsConstructor
public enum TenderStatus implements IStatus<Integer, String> {

    TO_BE_SUBMIT_PURCHASE_INFO(ProjectStatus.ON_PURCHASING, 101, "待填写采购信息"),
    TO_BE_SUBMIT_CONSTRUCTION_INFO(ProjectStatus.ON_PURCHASING, 102, "待填写合同信息"),
    TO_BE_SUBMIT_OPERATION_PLAN(ProjectStatus.ON_PURCHASING, 103, "待填写实施计划"),
    TO_BE_SUBMIT_FIRST_INSPECTED_INFO(ProjectStatus.TO_BE_FIRST_INSPECTED, 201, "待上传初验材料"),
    TO_BE_SUBMIT_FINALLY_INSPECTED_APPLY(ProjectStatus.ON_PILOT_RUNNING, 401, "待终验申报"),
    ON_FINALLY_INSPECTED_APPLY(ProjectStatus.ON_FINALLY_INSPECTED, 501, "终验申报中"),
    FINALLY_INSPECTED_FAILED(ProjectStatus.ON_FINALLY_INSPECTED, 502, "终验申报不通过"),
    FINALLY_INSPECTED_PASSED(ProjectStatus.ACCEPTED, 601, "终验完成");

    private final ProjectStatus projectStatus;
    private final Integer code;
    private final String desc;


    private static Optional<TenderStatus> get(Integer tenderStatus) {
        return Arrays.stream(values())
                .filter(w -> w.eq(tenderStatus))
                .findFirst();
    }

    public static TenderStatus getNoNull(Integer tenderStatus) {
        return get(tenderStatus).orElseThrow(() -> BizException.wrap("无效的标段状态：%s", tenderStatus));
    }

    public static String getDescByStatus(Integer tenderStatus) {
        return get(tenderStatus).flatMap(w -> Optional.of(w.getDesc())).orElse(StrUtil.EMPTY);
    }

}
