package com.hz.pm.api.projectlib.model.enumeration.status;

import cn.hutool.core.util.StrUtil;
import com.ningdatech.basic.exception.BizException;
import lombok.AllArgsConstructor;
import lombok.Getter;

import java.util.Arrays;
import java.util.Optional;

/**
 * <p>
 * SelfTestStatusEnum
 * </p>
 *
 * @author WendyYang
 * @since 10:16 2024/3/27
 */
@Getter
@AllArgsConstructor
public enum TenderTestValidStatus implements IStatus<Integer, String> {

    WITHOUT_TEST_VALID_INFO(3100, "待填写测试验证信息"),
    TEST_VALID_INFO_AUDIT(3101, "测试验证信息审核中"),
    TEST_VALID_INFO_FAILED(3102, "测试验证信息审核失败"),
    TEST_VALID_INFO_PASSED(3103, "测试验证信息审核通过");

    private final Integer code;
    private final String desc;

    private static Optional<TenderTestValidStatus> get(Integer code) {
        return Arrays.stream(values()).filter(w -> w.eq(code)).findFirst();
    }

    public static TenderTestValidStatus getNoNull(Integer code) {
        return get(code).orElseThrow(() -> BizException.wrap("无效的测试验证状态：%s", code));
    }

    public static String getDesc(Integer code) {
        return get(code).flatMap(w -> Optional.of(w.getDesc())).orElse(StrUtil.EMPTY);
    }

}
