package com.hz.pm.api.projectlib.service;

import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.core.toolkit.Assert;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.IService;
import com.hz.pm.api.common.model.constant.BizConst;
import com.hz.pm.api.projectdeclared.utils.ProjectIdCodeCacheUtil;
import com.hz.pm.api.projectlib.model.entity.Project;
import com.hz.pm.api.projectlib.model.po.ProjectPO;
import com.hz.pm.api.projectlib.model.req.ProjectListReq;

import java.util.Collection;
import java.util.Collections;
import java.util.List;
import java.util.Optional;

/**
 * <p>
 * 服务类
 * </p>
 *
 * @author WendyYang
 * @since 2023-02-03
 */
public interface IProjectService extends IService<Project> {

    default Optional<String> getProjectCode(Long projectId) {
        return Optional.ofNullable(ProjectIdCodeCacheUtil.get(projectId));
    }

    default Project getNoNull(Long projectId) {
        Project project = getById(projectId);
        Assert.notNull(project, "项目不存在");
        return project;
    }

    @Override
    default boolean save(Project entity) {
        boolean state = IService.super.save(entity);
        // 将项目ID与编码的对应关系保存下来
        ProjectIdCodeCacheUtil.put(entity.getId(), entity.getProjectCode());
        return state;
    }

    default Optional<Long> getNewProjectId(String projectCode) {
        return Optional.ofNullable(ProjectIdCodeCacheUtil.newest(projectCode));
    }

    default List<Long> allVersionProjectIds(String projectCode) {
        return ProjectIdCodeCacheUtil.get(projectCode);
    }

    default List<Long> allVersionProjectIds(Long projectId) {
        String projectCode = ProjectIdCodeCacheUtil.get(projectId);
        if (StrUtil.isBlank(projectCode)) {
            return Collections.emptyList();
        }
        return ProjectIdCodeCacheUtil.get(projectCode);
    }

    Project getNewProject(Long projectId);

    Project getNewestNoNull(Long projectId);

    Project getProjectByCode(String projectCode);

    Project getNoNull(String projectCode);

    Page<ProjectPO> pageAllWithPermission(Page<ProjectPO> page, ProjectListReq req);

    Page<ProjectPO> pageLibWithPermission(Page<ProjectPO> page, ProjectListReq req);

    default void reverseNewest(String projectCode, Long projectId) {
        LambdaUpdateWrapper<Project> wrapper = Wrappers.lambdaUpdate(Project.class)
                .set(Project::getNewest, Boolean.FALSE)
                .eq(Project::getNewest, Boolean.TRUE)
                .eq(Project::getProjectCode, projectCode)
                .ne(Project::getId, projectId);
        update(wrapper);
    }

    default List<Project> listNewestByProjectCodes(Collection<String> projectCode) {
        Wrapper<Project> query = Wrappers.lambdaQuery(Project.class)
                .eq(Project::getNewest, Boolean.TRUE)
                .in(Project::getProjectCode, projectCode);
        return list(query);
    }

    default Project getByProjectName(String projectName) {
        LambdaQueryWrapper<Project> query = Wrappers.lambdaQuery(Project.class)
                .eq(Project::getProjectName, projectName)
                .orderByDesc(Project::getVersion)
                .last(BizConst.LIMIT_1);
        return getOne(query);
    }

}
