package com.hz.pm.api.projectlib.service;

import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.IService;
import com.hz.pm.api.common.model.constant.BizConst;
import com.hz.pm.api.common.statemachine.event.ProjectStateChangeEvent;
import com.hz.pm.api.common.util.BizUtils;
import com.hz.pm.api.projectlib.model.entity.ProjectStatusChange;

import java.util.Collection;
import java.util.Comparator;
import java.util.Map;

/**
 * <p>
 * 项目状态变更 - 服务类
 * </p>
 *
 * @author CMM
 * @since 2023-02-27
 */
public interface IProjectStatusChangeService extends IService<ProjectStatusChange> {

    default ProjectStatusChange getLastOne(ProjectStateChangeEvent event, Long projectId) {
        Wrapper<ProjectStatusChange> query = Wrappers.lambdaQuery(ProjectStatusChange.class)
                .eq(ProjectStatusChange::getEvent, event)
                .eq(ProjectStatusChange::getProjectId, projectId)
                .orderByDesc(ProjectStatusChange::getCreateOn)
                .last(BizConst.LIMIT_1);
        return getOne(query);
    }

    default ProjectStatusChange getLastOne(ProjectStateChangeEvent event, Collection<Long> projectIds) {
        Wrapper<ProjectStatusChange> query = Wrappers.lambdaQuery(ProjectStatusChange.class)
                .in(ProjectStatusChange::getProjectId, projectIds)
                .eq(ProjectStatusChange::getEvent, event)
                .orderByDesc(ProjectStatusChange::getCreateOn)
                .last(BizConst.LIMIT_1);
        return getOne(query);
    }

    default ProjectStatusChange getLastOne(Collection<Long> projectIds, Collection<ProjectStateChangeEvent> events) {
        LambdaQueryWrapper<ProjectStatusChange> pscQuery = Wrappers.lambdaQuery(ProjectStatusChange.class)
                .in(ProjectStatusChange::getProjectId, projectIds)
                .in(ProjectStatusChange::getEvent, events)
                .orderByDesc(ProjectStatusChange::getCreateOn)
                .last(BizConst.LIMIT_1);
        return getOne(pscQuery);
    }

    default ProjectStatusChange getLastOne(String projectCode, Collection<ProjectStateChangeEvent> events) {
        LambdaQueryWrapper<ProjectStatusChange> pscQuery = Wrappers.lambdaQuery(ProjectStatusChange.class)
                .eq(ProjectStatusChange::getProjectCode, projectCode)
                .in(ProjectStatusChange::getEvent, events)
                .orderByDesc(ProjectStatusChange::getCreateOn)
                .last(BizConst.LIMIT_1);
        return getOne(pscQuery);
    }

    default Map<ProjectStateChangeEvent, ProjectStatusChange> listLastEventMap(String projectCode) {
        Wrapper<ProjectStatusChange> query = Wrappers.lambdaQuery(ProjectStatusChange.class)
                .eq(ProjectStatusChange::getProjectCode, projectCode)
                .orderByDesc(ProjectStatusChange::getCreateOn);
        return BizUtils.groupFirstMap(list(query),
                w -> ProjectStateChangeEvent.valueOf(w.getEvent()),
                Comparator.comparing(ProjectStatusChange::getCreateOn).reversed());
    }

}
