package com.hz.pm.api.sys.manage;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.IdUtil;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.hz.pm.api.common.enumeration.ProjectProcessType;
import com.ningdatech.basic.util.CollUtils;
import com.wflow.bean.dto.WflowModelHistorysInsertDto;
import com.wflow.bean.entity.WflowModels;
import com.wflow.workflow.bean.process.ProcessNode;
import com.wflow.workflow.service.ProcessModelService;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import java.util.Collections;
import java.util.List;
import java.util.Map;
import java.util.Set;

/**
 * <p>
 * InitProcessManage
 * </p>
 *
 * @author WendyYang
 * @since 20:45 2024/3/6
 */
@Slf4j
@Component
@RequiredArgsConstructor
public class InitProcessManage {


    private final ProcessModelService processModelService;

    @Transactional(rollbackFor = Exception.class)
    public void initProcessForUnits(Set<Long> unitIds) {
        if (CollUtil.isEmpty(unitIds)) {
            return;
        }
        List<String> strUnitIds = CollUtils.convert(unitIds, String::valueOf);
        ProjectProcessType[] stages = ProjectProcessType.values();
        List<WflowModels> allModels = processModelService.list(Wrappers.lambdaQuery(WflowModels.class)
                .in(WflowModels::getRegionCode, strUnitIds));
        Map<Integer, List<WflowModels>> modelsByType = CollUtils.group(allModels, WflowModels::getProcessType);
        for (ProjectProcessType stage : stages) {
            List<WflowModels> currModels = modelsByType.getOrDefault(stage.getCode(), Collections.emptyList());
            Map<String, WflowModels> modelsByUnitId = CollUtils.listToMap(currModels, WflowModels::getRegionCode);
            for (String unitId : strUnitIds) {
                if (modelsByUnitId.containsKey(unitId)) {
                    continue;
                }
                WflowModelHistorysInsertDto models = new WflowModelHistorysInsertDto();
                models.setGroupId(1);
                models.setProcessDefId("pd" + IdUtil.objectId());
                models.setFormName(stage.getDesc());
                models.setProcessType(stage.getCode());
                models.setRegionCode(unitId);
                models.setProcess(new ProcessNode<>());
                models.setSettings("{\"sign\":false,\"admin\":[],\"notify\":{},\"commiter\":[]}");
                models.setFormItems("[]");
                String formId = processModelService.insertProcess(models);
                processModelService.deployProcess(formId, null);
            }
        }
    }

}
