package com.hz.pm.api.sys.manage;

import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.toolkit.Assert;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.hz.pm.api.common.enumeration.ProjectProcessType;
import com.hz.pm.api.user.helper.MhUnitCache;
import com.ningdatech.basic.util.CollUtils;
import com.wflow.bean.entity.WflowModels;
import com.wflow.workflow.service.ProcessModelService;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;

import java.util.List;

/**
 * <p>
 * WflowModelManage
 * </p>
 *
 * @author WendyYang
 * @since 15:07 2024/3/5
 */
@Slf4j
@Component
@RequiredArgsConstructor
public class ProcessModelManage {

    private final MhUnitCache mhUnitCache;
    private final ProcessModelService processModelService;

    /**
     * 获取单位对应的流程配置
     *
     * @param processStage 流程类型
     * @param unitId       单位ID
     * @return 流程配置
     */
    private WflowModels getWflowModels(ProjectProcessType processStage, Long unitId) {
        List<Long> unitIdPaths = mhUnitCache.getUnitIdPaths(unitId);
        if (unitIdPaths.isEmpty()) {
            log.error("该单位无法进行项目申报：{}", unitId);
            return null;
        }
        List<String> unitIdPathsStr = CollUtils.convert(unitIdPaths, String::valueOf);
        String orderSql = String.format(" order by field(region_code, %s) desc limit 1",
                CollUtils.join(unitIdPaths, w -> "'" + w + "'", StrUtil.COMMA));
        return processModelService.getOne(Wrappers.lambdaQuery(WflowModels.class)
                .eq(WflowModels::getProcessType, processStage.getCode())
                .in(WflowModels::getRegionCode, unitIdPathsStr)
                .last(orderSql));
    }

    /**
     * 获取单位对应的流程配置
     *
     * @param processStage 流程类型
     * @param unitId       单位ID
     * @return 流程配置
     */
    public WflowModels getWflowModelsNoNull(ProjectProcessType processStage, Long unitId) {
        WflowModels wflowModels = getWflowModels(processStage, unitId);
        Assert.notNull(wflowModels, "%s 不存在", processStage.getDesc());
        return wflowModels;
    }

}
