package com.hz.pm.api.user.security.auth.credential;

import com.hz.pm.api.sms.constant.VerificationCodeType;
import com.hz.pm.api.sms.helper.VerifyCodeCheckHelper;
import com.hz.pm.api.user.model.enumeration.LoginTypeEnum;
import com.hz.pm.api.user.security.auth.constants.SessionTimeConst;
import com.hz.pm.api.user.security.auth.constants.UserDetailsServiceConstant;
import com.hz.pm.api.user.security.validate.CommonLoginException;
import com.ningdatech.basic.exception.BizException;
import org.springframework.security.authentication.AuthenticationProvider;
import org.springframework.security.authentication.BadCredentialsException;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.AuthenticationException;
import org.springframework.security.core.userdetails.UserDetails;
import org.springframework.security.core.userdetails.UserDetailsService;
import org.springframework.security.crypto.password.PasswordEncoder;

/**
 * @Author LiuXinXin
 * @Date 2020/8/3 8:55 下午
 * @Version 1.0
 **/
public class CredentialAuthProvider implements AuthenticationProvider {

    private UserDetailsService userDetailsService;
    private PasswordEncoder passwordEncoder;
    private Boolean phoneVerifyCodeSkip;

    private VerifyCodeCheckHelper verifyCodeCheckHelper;


    @Override
    public Authentication authenticate(Authentication authentication) throws AuthenticationException {
        if (!(authentication instanceof CredentialAuthToken)) {
            throw new BizException("CustomAuthProvider 只支持 CustomAuthToken");
        }
        CredentialAuthToken authenticationToken = (CredentialAuthToken) authentication;
        String principal = (String) authenticationToken.getPrincipal();

        UserDetails user = null;
        LoginTypeEnum loginTypeEnum = authenticationToken.getLoginTypeEnum();
        String credentials = (String) authenticationToken.getCredentials();
        switch (loginTypeEnum) {
            case PHONE_VERIFICATION_CODE_LOGIN: {
                if (Boolean.FALSE.equals(phoneVerifyCodeSkip)) {
                    // 校验短信验证码
                    boolean verificationResult = verifyCodeCheckHelper.verification(VerificationCodeType.LOGIN, principal, credentials);
                    if (!verificationResult && !SessionTimeConst.UNIVERSAL_VERIFICATION_CODE.equals(credentials)) {
                        throw new CommonLoginException("验证码错误");
                    }
                }
                user = userDetailsService.loadUserByUsername(principal + UserDetailsServiceConstant.USER_DETAILS_SERVICE_SEPARATOR + loginTypeEnum.name());
            }
            break;
            case USERNAME_PASSWORD_LOGIN: {
                user = userDetailsService.loadUserByUsername(principal + UserDetailsServiceConstant.USER_DETAILS_SERVICE_SEPARATOR + loginTypeEnum.name());
            }
            break;
        }
        // 将用户定义的user放入token中，这样可以在session中查询到所有自定义的用户信息
        return new CredentialAuthToken(user, user.getPassword(), user.getAuthorities());
    }

    protected void additionalAuthenticationChecks(UserDetails userDetails, CredentialAuthToken authentication)
            throws AuthenticationException {
        if (authentication.getCredentials() == null) {
            throw new BadCredentialsException("login fail! password is null");
        }
        String presentedPassword = authentication.getCredentials().toString();
        if (!passwordEncoder.matches(presentedPassword, userDetails.getPassword())) {
            throw new BadCredentialsException("login fail! password is error");
        }
    }


    @Override
    public boolean supports(Class<?> authentication) {
        return CredentialAuthToken.class.isAssignableFrom(authentication);
    }

    public void setUserDetailsService(UserDetailsService userDetailsService) {
        this.userDetailsService = userDetailsService;
    }

    public void setPasswordEncoder(PasswordEncoder passwordEncoder) {
        this.passwordEncoder = passwordEncoder;
    }

    public void setVerifyCodeCheckHelper(VerifyCodeCheckHelper verifyCodeCheckHelper) {
        this.verifyCodeCheckHelper = verifyCodeCheckHelper;
    }

    public void setPhoneVerifyCodeSkip(Boolean phoneVerifyCodeSkip) {
        this.phoneVerifyCodeSkip = phoneVerifyCodeSkip;
    }

}
